package dnibuilder

import DNImaster.MasterFactory
import DNImaster.NetworkInfrastructure
import java.util.HashSet
import java.util.LinkedList
import DNImaster.NetworkConfiguration
import DNImaster.Link
import DNImaster.NetworkInterface
import DNImaster.Node
import DNImaster.PhysicalNetworkInterface
import DNImaster.impl.EndNodePerfSpecImpl
import DNImaster.impl.IntermediateNodePerfSpecImpl
import DNImaster.GenericWorkload
import DNImaster.SoftwareComponent
import DNIunits.Speed
import DNIunits.UnitPrefix
import DNIunits.UnitsFactory
import DNIunits.Dependency
import DNImaster.IntermediateNodePerfSpec
import java.util.List
import DNImaster.NetworkInterfacePerfSpec
import DNImaster.Route
import java.util.Set

class NetworkTrafficModelInstanceGenerator {
	
	extension UnitsFactory = UnitsFactory.eINSTANCE
	extension MasterFactory = MasterFactory.eINSTANCE
	
	NetworkInfrastructure ns
	
	int id = 0 
	
	def genId(){
		id++
		return "id"+id
	}
	
	def init(CaseStudy cs) {
		createNetworkInfrastructure => [

			dataCenterName = "Extracted_from_"+cs.orgPcapngFileNamePrefix
			dataCenterAltName = "abc"
			configuration  = createNetworkConfiguration => [addProtocols]
			structure      = createNetworkStructure
			traffic        = createNetworkTraffic
		]
	}
	
	def NetworkInfrastructure generate(CaseStudy cs) {

		 ns = init(cs)
		 ns = extractSetOfNodes(cs, ns)
		 ns.traffic = extraxtSoftwareComponents(cs, ns)
		 extraxtWorkloads(cs, ns)
		 buildTheRest(cs, ns)
		 
		
		 
		 ns
	}
	
	
	def String getLeftIP(IPPair ipp){
		getIPOutOfFlowName(0, ipp.name)	
	}
	
	def String getRightIP(IPPair ipp){
		getIPOutOfFlowName(1, ipp.name)	
	}
	
	def getIPOutOfFlowName(int i, String name){
		name.split('_data_').get(1).split('.csv.list').get(0).split('-').get(i)
	}
	
	def NetworkInfrastructure extractSetOfNodes(CaseStudy cs, NetworkInfrastructure nis){
		//partial support of structure generation - based on guessing - without topology information, only guessing is possible
		//assumption: node has one interface => 1 IP = 1 node
		println("==== Creating nodes")
		for(ippair: cs.ippairs){	
			val lip = getLeftIP(ippair) 
			val rip = getRightIP(ippair)
			
			
			val nodenameL = "node_with_IP_"+lip
			val nodenameR = "node_with_IP_"+rip
			
			
			
			
			if(nis.structure.nodes.filter[name.equals(nodenameL)].size==0){
				val node = createServerNode(nodenameL, nis)
				println("Creating node named: "+nodenameL)
				node.interfaces.get(0).addresses += createEntityAddress => [
					address = lip
					addressGivenBy = nis.configuration.protocolsAndStacks.protocols.findFirst[name.startsWith("IP")]
				]
				nis.structure.nodes.add(node as Node)
			}
			if(nis.structure.nodes.filter[name.equals(nodenameR)].size==0){
				val node = createServerNode(nodenameR, nis)
				println("Creating node named: "+nodenameR)
				node.interfaces.get(0).addresses += createEntityAddress => [
					address = rip
					addressGivenBy = nis.configuration.protocolsAndStacks.protocols.findFirst[name.startsWith("IP")]
				]
				nis.structure.nodes.add(node as Node)
			}
		}
		println("==== Creating nodes done")
		return nis	
	}
	
	def createServerNode(String nodename, NetworkInfrastructure nis){
		//first create generic network interface
		val PhysicalNetworkInterface netif = createPhysicalNetworkInterface => [
				id = genId
				name = "eth0"
				usedProtocolStack = nis.configuration.protocolsAndStacks.stacks.findFirst[name.startsWith("TCP")]
				performance = createPhysicalNetworkInterfacePerfSpec => [
					isUp = true
					packetProcessingTime = (createConstantDoubleVariable => [value = 0.00001 ]) as Dependency   //dummy values FIXME
					MTU = 1500
					interfaceSpeed = createSpeedUnit => [
						value = 1 //dummy values FIXME
						prefix = UnitPrefix.G
						unit = Speed.BITS_PER_SEC
					]
				]
				addresses += createEntityAddress => [
					address = "00:AC:EF:AC:E2:FA:CF"
					addressGivenBy = nis.configuration.protocolsAndStacks.protocols.findFirst[name.startsWith("ETH")]
				]					
			]
		//then create node and add this interface to the node
		val node = createPhysicalNode  => [
				id = genId
				name = nodename
				performance = createEndNodePerfSpec => [
					softwareLayersDelay = createConstantDoubleVariable => [value = 0.00001 ]  //dummy values FIXME
					
					//TODO: conversion to dnitxt disallows this here - why?
					dataplane = createDataPlanePerfSpec =>[
						forwardingLatency = createConstantDoubleVariable => [value = 0.0000001 ]  //dummy values FIXME
						forwardingBandwidthPPS = createConstantLongVariable => [value = 100000000 ]  //dummy values FIXME
						forwardingBandwidthBPS = createConstantLongVariable => [value = 1000000000 ]  //dummy values FIXME
						packetProcessingTime = createConstantDoubleVariable => [value = 0.00001 ]  //dummy values FIXME
					]
					controlplane = createControlPlanePerfSpec =>[
						processingLatency = createConstantDoubleVariable => [value = 0.00001 ]  //dummy values FIXME
						processingBandwidthReqPS = createConstantLongVariable => [value = 100000000 ]  //dummy values FIXME
						packetProcessingTime = createConstantDoubleVariable => [value = 0.00001 ]  //dummy values FIXME
					]
				]
				interfaces.add(netif)
			]
		return node
	}
	
	// reads the text file and calculates the number of Flows that should be created to represent different sizes of messages
	def extraxtFlows(CaseStudy cs, IPPair wavelet_flow, SoftwareComponent destinationSC, SoftwareComponent sourceSC ){
		
		val uniqueDataSizes = new HashSet<Integer>()
		val flows = new LinkedList()
	
		
		
		for (generator: wavelet_flow.generators){
			uniqueDataSizes.add(generator.value)
		}
		for(uds: uniqueDataSizes){
			val dniflow = createGenericFlow => [
				id = genId
				destinationSoftwareComponent = destinationSC
				sourceSoftwareComponent = sourceSC
				dataSize = (createConstantLongVariable => [value = uds]) as Dependency
				name = wavelet_flow.name+"_size_"+uds
				alternativeNames += "wavelet_gen_id_"+wavelet_flow.name
			]
			flows += dniflow
		}
		
		flows
	}
	
	def extraxtSoftwareComponents(CaseStudy cs, NetworkInfrastructure nis ){
		
		println("==== Creating Software components")
		
		for(ippair: cs.ippairs){	
			println("== Processing ippair: "+ippair.name+". it has "+ippair.generators.size+" generators")
			
			val destinationSoftwareComponent = createSoftwareComponent => [ s |
				s.id = genId
				val candidate_nodes = nis.structure.nodes.filter[name.endsWith(getRightIP(ippair))]
				s.name = "destination_w_ip_"+getRightIP(ippair)
				if(candidate_nodes.size > 0){
					s.deployedOn = candidate_nodes.head as Node	
					println("#### SoftComponent "+s.name+" ID "+id+" will be deployed on node "+s.deployedOn)
				}
				else{
					println("#### SoftComponent "+s.name+" ID "+id+" NO CANDIDATE NODE TO DEPLOY ON")
				}
				
				
			]
			println("Creating Software component "+destinationSoftwareComponent.name)
						
			val sourceSoftwareComponent = createSoftwareComponent => [
				id = genId				
				val candidate_nodes = nis.structure.nodes.filter[name.endsWith(getLeftIP(ippair))]
				if(candidate_nodes.size > 0){
					deployedOn = candidate_nodes.head as Node	
					println("#### SoftComponent "+name+" ID "+id+" will be deployed on node "+deployedOn)
				}
				name = "source_w_ip_"+getLeftIP(ippair)
			]
			println("Creating Software component "+sourceSoftwareComponent.name)
			
			val dni_flows = extraxtFlows(cs, ippair, destinationSoftwareComponent, sourceSoftwareComponent)
			for(dniflow: dni_flows){
				println("\t Creating Flow "+dniflow.name)
				nis.traffic.flows.add(dniflow)
			}
			
			val ts = createTrafficSource => [
					id = genId
					name = "empty"
					softwareComponent = sourceSoftwareComponent
					workload.add(createGenericWorkload)
				]
			sourceSoftwareComponent.trafficSources.add(ts)
			nis.traffic.software.add(destinationSoftwareComponent)
			nis.traffic.software.add(sourceSoftwareComponent)
			println("== Software component added")
		}
		println("==== Creating Software components done")
		return nis.traffic
	}
	
	def extraxtWorkloads(CaseStudy cs, NetworkInfrastructure nis ){

		println("==== Creating Workloads")
		
		for(ippair: cs.ippairs.filter[generators.size>0]){
			//single csv.list file
			val sender_cs_ip = getLeftIP(ippair)
			val receiver_cs_ip = getRightIP(ippair)
			
			//get single flow that connects these two addresses
			val example_flows = nis.traffic.flows.filter[getIPOutOfFlowName(0,name)==sender_cs_ip && getIPOutOfFlowName(1,name)==receiver_cs_ip ]
			val example_flow = example_flows.get(0)
			
			val sender_softwareComponent = example_flow.sourceSoftwareComponent
			val receiver_softwareComponent = example_flow.destinationSoftwareComponent
			
			println("\t ==== Checking software components")
			//we expect, that this sender software component has only single traffic source
			if(sender_softwareComponent.trafficSources.size>1 || sender_softwareComponent.trafficSources==0){
				println("!!!ALARM: TODO! soft component has no tses or too many tses")
			}
			
			val ts = sender_softwareComponent.trafficSources.get(0)
			if(ts.workload.size!=1){
				println("!!!ALARM: TODO! TS has more than one workload")
			}
			val wrk = ts.workload.get(0) as GenericWorkload
			//we expect, that this ts has no actions in workload
			if( wrk.actions.size!=0  ){
				println("!!!ALARM: TODO! TS has workload with existing actions!")
			}
			println("\t ==== Checking software components done")
			println("\t ==== Extracting workload")
			
			val branch = createBranchAction
			wrk.actions.add(createSequenceAction => [
				intern.add(createStartAction)
				intern.add(branch)
				intern.add(createStopAction)
				])
			for(gen: ippair.generators){
				/*
				Generator [
				  name = "genname"
				  scale = 0
				  value = 500
				  begin = 1.0
				  end = 6.0
				]
				*/
				println("\t --------------------")
				println("\t Processing gen: "+gen)
				val the_flow = example_flows.filter[name==ippair.name+"_size_"+gen.value].get(0)
				println("\t Found flow: "+the_flow.name)
				
				//first, the generator waits
				val initial_wait = createWaitAction => [
						waitTime = (createConstantDoubleVariable => [value = gen.begin]) as Dependency
				]
				//then a loop is created //ACHTUNG! highest sensitivity of the model is here! By high prediction errors - check this place first
				val loop = createLoopAction => [
						intern = createSequenceAction =>[
							intern.add( createTransmitAction => [ 
								flow = the_flow
							])
							intern.add(createWaitAction => [
								waitTime = (createConstantDoubleVariable => [value = Math.pow(2, gen.scale)]) as Dependency
							])
						]
						val calculated_num_iter = calculateNumIterations(gen) 
//						println("\t NumIter="+calculated_num_iter+" for "+gen)
						numIterations = (createConstantLongVariable => [value = calculated_num_iter]) as Dependency
				] 
				
				val seq = createSequenceAction => [ 
					intern.add(initial_wait)
					intern.add(loop)
				]
				branch.intern.add(seq)
				
				
			}
			println("\t ==== Extracting workload done")
		}
		
		println("==== Creating Workloads done")
		
//		println(cs)
	}
	
	def calculateNumIterations(Generator gen) {

		val duration = gen.end - gen.begin
//		println("\t\t duration= "+duration)
		val interval = Math.pow(2, gen.scale)
//		println("\t\t interval= "+interval+" (for scale "+gen.scale+")")
		
		if(interval > duration){
			return 1 as long
		}
		if(interval == duration){
			return 2 as long
		}
		val numIter = 1 + duration as double / interval as double 
		
		return numIter as long
	}
	
	///////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////////// 
	///////////////////////////////////////////////////////////////////////
	// Immitation of manual modeling
	
	def buildTheRest(CaseStudy study, NetworkInfrastructure infrastructure) {
		
		//empty routing
		ns.configuration.routes = createRoutesRepository 
		 
		buildTopology(infrastructure)
		buildRouting(infrastructure)
		

	}
	
	///////////////////////////////////////////////////////////////////////
	
	def buildRouting(NetworkInfrastructure inf) {
		
		println("==== Building Routes")
		val flows = inf.traffic.flows;
//		val software = inf.traffic.software;
		  
		for(f: flows){
			val sourceSoftware = f.sourceSoftwareComponent;
			val trafficSources = sourceSoftware.trafficSources;
			if(trafficSources.size>0 && trafficSources.head.workload.size>0){
				val sourceNode = sourceSoftware.deployedOn;
				val destinationNode = f.destinationSoftwareComponent.deployedOn;
				
				val route = createRouteForNodes(inf, sourceNode,destinationNode);
				inf.configuration.routes.routes += route
			}
		}
		//removing duplicate routes
		var r1s = inf.configuration.routes.routes.toSet
		var Set<Route> toRemove = new HashSet
		for(r1: r1s){
			for(r2: r1s){
				if(r1.id != r2.id && routesIdentic(r1,r2) && !toRemove.contains(r1) && !toRemove.contains(r2)){
					println("\t Removing duplicate route "+r1.id+" - is identic with "+r2.id)
					toRemove.add(r1)
				}
			}
		}
		for(r: toRemove){
			inf.configuration.routes.routes.remove(r)
		}
		println("==== Building Routes done")
	}
	///////////////////////////////////////////////////////////////////////
	
	def routesIdentic(Route r1, Route r2) {
//		println("Comparing routes "+r1.id+" with "+r2.id)
		
		if(r1.start.id != r2.start.id){
//			println("\t start iface different")
			return false;
		}
		if(r1.end.id != r2.end.id){
//			println("\t end iface different")
			return false;
		}
		if(r1.hops != null && r2.hops != null){
			var hop1 = r1.hops
			var hop2 = r2.hops
//			println("\t A")
			if(hop1.interfaceref.id != hop2.interfaceref.id){
//				println("\t first hop different")
				return false;
			}
//			println("\t B")
			while(hop1.nextHop != null && hop2.nextHop != null){
				hop1 = hop1.nextHop
				hop2 = hop2.nextHop
				if(hop1.interfaceref.id != hop2.interfaceref.id){
//					println("\t futher hop different")
					return false;
				}
			}
			
			//end of while - some of hops' nextHop was null
			if(hop1.nextHop == null &&  hop2.nextHop != null || hop1.nextHop != null &&  hop2.nextHop == null){
//				println("Routes have different length")
				return false;
			} 
			if(hop1.nextHop == null &&  hop2.nextHop == null && hop1.interfaceref.id == r1.end.id && hop2.interfaceref.id == r2.end.id && r1.end.id == r2.end.id){
//				println("\t THEY ARE IDENTIC!")
				return true
			}
		}
		println("\t shall never get here")
		return false
	}
	///////////////////////////////////////////////////////////////////////
	
	def createRouteForNodes(NetworkInfrastructure inf, Node startNode, Node endNode) {
		//we assume, that every node has only one interface that is connected. 
		//in reality, the info about routing should be extracted as well
		
		//select active interfaces
		var NetworkInterface if1;
		var NetworkInterface if2;
		
		val Iterable<NetworkInterface> startNodeIfaces = startNode.interfaces.filter[i|(i.performance as NetworkInterfacePerfSpec).isUp==true];
		val Iterable<NetworkInterface> endNodeIfaces = endNode.interfaces.filter[i|(i.performance as NetworkInterfacePerfSpec).isUp==true];
		
		
		if(startNodeIfaces.size==1 && endNodeIfaces.size==1){
			val route = createRoute => [
				start = startNodeIfaces.head
				end = endNodeIfaces.head
				name = "Route"
				id = genId
				alternativeNames += "Route_from_node_"+startNode.name+"_if_"+startNodeIfaces.head.name+"_to_"+endNode.name+"_if_"+endNodeIfaces.head.name;
				name = alternativeNames.head 
			]
			
			val kabel = inf.structure.links.filter[l|l.connects.contains(startNodeIfaces.head)].head;
			val hop = createHop
			
			if(kabel != null){
				val end1 = kabel.connects.get(0)
				val end2 = kabel.connects.get(1)
				
				var NetworkInterface nextIface = end1;
				if(end1 == startNodeIfaces.head){
					nextIface = end2;
				}
				 
				//if this is direct connection
				if(end1 == startNodeIfaces.head && end2 == endNodeIfaces.head){
					route.hops = createHop => [interfaceref = end2]
//					println("returning direct connection route")
					return route
				}
				if(end2 == startNodeIfaces.head && end1 == endNodeIfaces.head){
					route.hops = createHop => [interfaceref = end1]
//					println("returning direct connection route")
					return route
				}
				
				//there are nodes between start and end!
				hop.interfaceref = nextIface //nextIface - interface of the switch
				route.hops = hop
			}
			//taking the perspective of the end of the route
			val endKabel = inf.structure.links.filter[l|l.connects.contains(endNodeIfaces.head)].head;	
			if(endKabel != null){
				val end1 = endKabel.connects.get(0)
				val end2 = endKabel.connects.get(1)
				var NetworkInterface outgoingSwitchInterface = end1;
				if(end1 == endNodeIfaces.head){
					outgoingSwitchInterface = end2;
				}
				//adding the hop to the route
				
				if(outgoingSwitchInterface == end1){ //needs to be done like this because of var/val difference
					route.hops.nextHop = createHop => [interfaceref = end1]	
					route.hops.nextHop.nextHop = createHop => [interfaceref = end2]
				}
				else{
//					println(route)
//					println(route.hops)
//					println(route.hops.nextHop)
					route.hops.nextHop = createHop => [interfaceref = end2]
					route.hops.nextHop.nextHop = createHop => [interfaceref = end1]
				}
//				println("returning non-direct connection route")
				return route
			}
		}
		println("returning empty route")
		return createRoute
	}

	
	///////////////////////////////////////////////////////////////////////
	
	def buildTopology(NetworkInfrastructure inf) {
		
		println("==== Building Topology")
		
		val servers = inf.structure.nodes.filter[node|node.performance.class == EndNodePerfSpecImpl]
		val switches = inf.structure.nodes.filter[node|node.performance.class == IntermediateNodePerfSpecImpl]
		
		
		//Assuming star topology - adding the star switch
		val starSw = createPhysicalNode  => [
				id = genId
				name = "theSwitch"
				performance = createIntermediateNodePerfSpec => [
					dataplane = createDataPlanePerfSpec =>[
						forwardingLatency = createConstantDoubleVariable => [value = 0.0000001 ]  //dummy values FIXME
						forwardingBandwidthPPS = createConstantLongVariable => [value = 100000000 ]  //dummy values FIXME
						forwardingBandwidthBPS = createConstantLongVariable => [value = 1000000000 ]  //dummy values FIXME
						packetProcessingTime = createConstantDoubleVariable => [value = 0.00001 ]  //dummy values FIXME
					]
					controlplane = createControlPlanePerfSpec =>[
						processingLatency = createConstantDoubleVariable => [value = 0.00001 ]  //dummy values FIXME
						processingBandwidthReqPS = createConstantLongVariable => [value = 100000000 ]  //dummy values FIXME
						packetProcessingTime = createConstantDoubleVariable => [value = 0.00001 ]  //dummy values FIXME
					]
				]
				

				
			]
		inf.structure.nodes.add(starSw as Node)
		
		//connect the nodes to the switch
		var i = 0
		for(s: servers){
			val interface = createPreconfiguredPhysicalNetworkInterface(inf,i)
			
						
			val serverInterface = s.interfaces.head as NetworkInterface
			starSw.interfaces += interface
			 
			var link = createPhysicalLink => [
				id = genId
				name = "kabel_switch_"+starSw.id+"_node_"+s.id
				connects += serverInterface as PhysicalNetworkInterface
				connects += interface
				performance = createPhysicalLinkPerfSpec => [
					 propagationDelay = createConstantDoubleVariable => [value = 5.7e-9 ]  //5.7 ns per meter 
				]
			]
			inf.structure.links += link as Link
			i=i+1
		}
		
		
		println("\tServers: "+servers)
		println("\tSwitches: "+switches)
		println("\tLinks: "+inf.structure.links)

		println("==== Building Topology done")
	}
	
	
	def createPreconfiguredPhysicalNetworkInterface(NetworkInfrastructure inf, int index){
		createPhysicalNetworkInterface => [
			id = genId
			name = "eth"+index
			usedProtocolStack = inf.configuration.protocolsAndStacks.stacks.findFirst[name.startsWith("TCP")]
			performance = createPhysicalNetworkInterfacePerfSpec => [
				isUp = true
				packetProcessingTime = createConstantDoubleVariable => [value = 0.00001 ]  //dummy values FIXME
				MTU = 1500
				interfaceSpeed = createSpeedUnit => [
					value = 1 //dummy values FIXME
					prefix = UnitPrefix.G
					unit = Speed.BITS_PER_SEC
				]
			]
			addresses += createEntityAddress => [
				address = "00:AC:EF:AC:E2:FA:CF"
				addressGivenBy = inf.configuration.protocolsAndStacks.protocols.findFirst[name.startsWith("ETH")]
			]					
		]
	}
	
	///////////////////////////////////////////////////////////////////////
	
	def addProtocols(NetworkConfiguration conf) {

		conf.protocolsAndStacks = createProtocolsRepository => [

			// layers ETH <- IP <- TCP
			val layerETH = createProtocolLayer => [
				protocol = createNetworkProtocol => [
					name = "ETH"
					dataUnitLength = 1500
					headersLength = 32
					connectionless = true
					deliveryGuaranteed = false
				]
				name = protocol.name
			]
			
			val layerIP = createProtocolLayer => [
				protocol = createNetworkProtocol => [
					name = "IP"
					dataUnitLength = 65000
					headersLength = 48
					connectionless = true
					deliveryGuaranteed = false
				]
				name = protocol.name
				isCarriedBy = layerETH
			]

			val layerTCP = createProtocolLayer => [
				protocol = createNetworkProtocol => [
					name = "TCP"
					dataUnitLength = 65000
					headersLength = 48
					connectionless = false
					deliveryGuaranteed = true
				]
				name = protocol.name
				isCarriedBy = layerIP
			]
			
			// adding protocols to repository
			protocols += layerETH.protocol
			protocols += layerIP.protocol
			protocols += layerTCP.protocol
			
			// for hosts
			stacks += createProtocolStack => [
				name = "TCP/IP/ETH"
				layers += layerTCP
				layers += layerIP
				layers += layerETH
			]

			// for routers
			stacks += createProtocolStack => [
				name = "IP/ETH"
				layers += layerIP
				layers += layerETH
			]
			
			// for switches
			stacks += createProtocolStack => [
				name = "ETH"
				layers += layerETH
			]
		]	
	}
}
