package dnibuilder;

import DNImaster.AbstractAction;
import DNImaster.BranchAction;
import DNImaster.ControlPlanePerfSpec;
import DNImaster.DataPlanePerfSpec;
import DNImaster.EndNodePerfSpec;
import DNImaster.EntityAddress;
import DNImaster.Flow;
import DNImaster.GenericFlow;
import DNImaster.GenericWorkload;
import DNImaster.Hop;
import DNImaster.IntermediateNodePerfSpec;
import DNImaster.Link;
import DNImaster.LinkPerfSpec;
import DNImaster.LoopAction;
import DNImaster.MasterFactory;
import DNImaster.NetworkConfiguration;
import DNImaster.NetworkInfrastructure;
import DNImaster.NetworkInterface;
import DNImaster.NetworkInterfacePerfSpec;
import DNImaster.NetworkProtocol;
import DNImaster.NetworkStructure;
import DNImaster.NetworkTraffic;
import DNImaster.Node;
import DNImaster.PerformanceSpecification;
import DNImaster.PhysicalLink;
import DNImaster.PhysicalLinkPerfSpec;
import DNImaster.PhysicalNetworkInterface;
import DNImaster.PhysicalNetworkInterfacePerfSpec;
import DNImaster.PhysicalNode;
import DNImaster.ProtocolLayer;
import DNImaster.ProtocolStack;
import DNImaster.ProtocolsRepository;
import DNImaster.Route;
import DNImaster.RoutesRepository;
import DNImaster.SequenceAction;
import DNImaster.SoftwareComponent;
import DNImaster.StartAction;
import DNImaster.StopAction;
import DNImaster.TrafficSource;
import DNImaster.TransmitAction;
import DNImaster.WaitAction;
import DNImaster.Workload;
import DNImaster.impl.EndNodePerfSpecImpl;
import DNImaster.impl.IntermediateNodePerfSpecImpl;
import DNIunits.ConstantDoubleVariable;
import DNIunits.ConstantLongVariable;
import DNIunits.Dependency;
import DNIunits.Speed;
import DNIunits.SpeedUnit;
import DNIunits.UnitPrefix;
import DNIunits.UnitsFactory;
import com.google.common.base.Objects;
import dnibuilder.CaseStudy;
import dnibuilder.Generator;
import dnibuilder.IPPair;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

@SuppressWarnings("all")
public class NetworkTrafficModelInstanceGenerator {
  @Extension
  private UnitsFactory _unitsFactory = UnitsFactory.eINSTANCE;
  
  @Extension
  private MasterFactory _masterFactory = MasterFactory.eINSTANCE;
  
  private NetworkInfrastructure ns;
  
  private int id = 0;
  
  public String genId() {
    this.id++;
    return ("id" + Integer.valueOf(this.id));
  }
  
  public NetworkInfrastructure init(final CaseStudy cs) {
    NetworkInfrastructure _createNetworkInfrastructure = this._masterFactory.createNetworkInfrastructure();
    final Procedure1<NetworkInfrastructure> _function = new Procedure1<NetworkInfrastructure>() {
      @Override
      public void apply(final NetworkInfrastructure it) {
        String _orgPcapngFileNamePrefix = cs.getOrgPcapngFileNamePrefix();
        String _plus = ("Extracted_from_" + _orgPcapngFileNamePrefix);
        it.setDataCenterName(_plus);
        it.setDataCenterAltName("abc");
        NetworkConfiguration _createNetworkConfiguration = NetworkTrafficModelInstanceGenerator.this._masterFactory.createNetworkConfiguration();
        final Procedure1<NetworkConfiguration> _function = new Procedure1<NetworkConfiguration>() {
          @Override
          public void apply(final NetworkConfiguration it) {
            NetworkTrafficModelInstanceGenerator.this.addProtocols(it);
          }
        };
        NetworkConfiguration _doubleArrow = ObjectExtensions.<NetworkConfiguration>operator_doubleArrow(_createNetworkConfiguration, _function);
        it.setConfiguration(_doubleArrow);
        NetworkStructure _createNetworkStructure = NetworkTrafficModelInstanceGenerator.this._masterFactory.createNetworkStructure();
        it.setStructure(_createNetworkStructure);
        NetworkTraffic _createNetworkTraffic = NetworkTrafficModelInstanceGenerator.this._masterFactory.createNetworkTraffic();
        it.setTraffic(_createNetworkTraffic);
      }
    };
    return ObjectExtensions.<NetworkInfrastructure>operator_doubleArrow(_createNetworkInfrastructure, _function);
  }
  
  public NetworkInfrastructure generate(final CaseStudy cs) {
    NetworkInfrastructure _xblockexpression = null;
    {
      NetworkInfrastructure _init = this.init(cs);
      this.ns = _init;
      NetworkInfrastructure _extractSetOfNodes = this.extractSetOfNodes(cs, this.ns);
      this.ns = _extractSetOfNodes;
      NetworkTraffic _extraxtSoftwareComponents = this.extraxtSoftwareComponents(cs, this.ns);
      this.ns.setTraffic(_extraxtSoftwareComponents);
      this.extraxtWorkloads(cs, this.ns);
      this.buildTheRest(cs, this.ns);
      _xblockexpression = this.ns;
    }
    return _xblockexpression;
  }
  
  public String getLeftIP(final IPPair ipp) {
    String _name = ipp.getName();
    return this.getIPOutOfFlowName(0, _name);
  }
  
  public String getRightIP(final IPPair ipp) {
    String _name = ipp.getName();
    return this.getIPOutOfFlowName(1, _name);
  }
  
  public String getIPOutOfFlowName(final int i, final String name) {
    String[] _split = name.split("_data_");
    String _get = _split[1];
    String[] _split_1 = _get.split(".csv.list");
    String _get_1 = _split_1[0];
    String[] _split_2 = _get_1.split("-");
    return _split_2[i];
  }
  
  public NetworkInfrastructure extractSetOfNodes(final CaseStudy cs, final NetworkInfrastructure nis) {
    InputOutput.<String>println("==== Creating nodes");
    List<IPPair> _ippairs = cs.getIppairs();
    for (final IPPair ippair : _ippairs) {
      {
        final String lip = this.getLeftIP(ippair);
        final String rip = this.getRightIP(ippair);
        final String nodenameL = ("node_with_IP_" + lip);
        final String nodenameR = ("node_with_IP_" + rip);
        NetworkStructure _structure = nis.getStructure();
        EList<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _nodes = _structure.getNodes();
        final Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean> _function = new Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean>() {
          @Override
          public Boolean apply(final Node<NetworkInterface<NetworkInterfacePerfSpec>> it) {
            String _name = it.getName();
            return Boolean.valueOf(_name.equals(nodenameL));
          }
        };
        Iterable<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _filter = IterableExtensions.<Node<NetworkInterface<NetworkInterfacePerfSpec>>>filter(_nodes, _function);
        int _size = IterableExtensions.size(_filter);
        boolean _equals = (_size == 0);
        if (_equals) {
          final PhysicalNode node = this.createServerNode(nodenameL, nis);
          InputOutput.<String>println(("Creating node named: " + nodenameL));
          EList<PhysicalNetworkInterface> _interfaces = node.getInterfaces();
          PhysicalNetworkInterface _get = _interfaces.get(0);
          EList<EntityAddress> _addresses = _get.getAddresses();
          EntityAddress _createEntityAddress = this._masterFactory.createEntityAddress();
          final Procedure1<EntityAddress> _function_1 = new Procedure1<EntityAddress>() {
            @Override
            public void apply(final EntityAddress it) {
              it.setAddress(lip);
              NetworkConfiguration _configuration = nis.getConfiguration();
              ProtocolsRepository _protocolsAndStacks = _configuration.getProtocolsAndStacks();
              EList<NetworkProtocol> _protocols = _protocolsAndStacks.getProtocols();
              final Function1<NetworkProtocol, Boolean> _function = new Function1<NetworkProtocol, Boolean>() {
                @Override
                public Boolean apply(final NetworkProtocol it) {
                  String _name = it.getName();
                  return Boolean.valueOf(_name.startsWith("IP"));
                }
              };
              NetworkProtocol _findFirst = IterableExtensions.<NetworkProtocol>findFirst(_protocols, _function);
              it.setAddressGivenBy(_findFirst);
            }
          };
          EntityAddress _doubleArrow = ObjectExtensions.<EntityAddress>operator_doubleArrow(_createEntityAddress, _function_1);
          _addresses.add(_doubleArrow);
          NetworkStructure _structure_1 = nis.getStructure();
          EList<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _nodes_1 = _structure_1.getNodes();
          _nodes_1.add(((Node) node));
        }
        NetworkStructure _structure_2 = nis.getStructure();
        EList<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _nodes_2 = _structure_2.getNodes();
        final Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean> _function_2 = new Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean>() {
          @Override
          public Boolean apply(final Node<NetworkInterface<NetworkInterfacePerfSpec>> it) {
            String _name = it.getName();
            return Boolean.valueOf(_name.equals(nodenameR));
          }
        };
        Iterable<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _filter_1 = IterableExtensions.<Node<NetworkInterface<NetworkInterfacePerfSpec>>>filter(_nodes_2, _function_2);
        int _size_1 = IterableExtensions.size(_filter_1);
        boolean _equals_1 = (_size_1 == 0);
        if (_equals_1) {
          final PhysicalNode node_1 = this.createServerNode(nodenameR, nis);
          InputOutput.<String>println(("Creating node named: " + nodenameR));
          EList<PhysicalNetworkInterface> _interfaces_1 = node_1.getInterfaces();
          PhysicalNetworkInterface _get_1 = _interfaces_1.get(0);
          EList<EntityAddress> _addresses_1 = _get_1.getAddresses();
          EntityAddress _createEntityAddress_1 = this._masterFactory.createEntityAddress();
          final Procedure1<EntityAddress> _function_3 = new Procedure1<EntityAddress>() {
            @Override
            public void apply(final EntityAddress it) {
              it.setAddress(rip);
              NetworkConfiguration _configuration = nis.getConfiguration();
              ProtocolsRepository _protocolsAndStacks = _configuration.getProtocolsAndStacks();
              EList<NetworkProtocol> _protocols = _protocolsAndStacks.getProtocols();
              final Function1<NetworkProtocol, Boolean> _function = new Function1<NetworkProtocol, Boolean>() {
                @Override
                public Boolean apply(final NetworkProtocol it) {
                  String _name = it.getName();
                  return Boolean.valueOf(_name.startsWith("IP"));
                }
              };
              NetworkProtocol _findFirst = IterableExtensions.<NetworkProtocol>findFirst(_protocols, _function);
              it.setAddressGivenBy(_findFirst);
            }
          };
          EntityAddress _doubleArrow_1 = ObjectExtensions.<EntityAddress>operator_doubleArrow(_createEntityAddress_1, _function_3);
          _addresses_1.add(_doubleArrow_1);
          NetworkStructure _structure_3 = nis.getStructure();
          EList<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _nodes_3 = _structure_3.getNodes();
          _nodes_3.add(((Node) node_1));
        }
      }
    }
    InputOutput.<String>println("==== Creating nodes done");
    return nis;
  }
  
  public PhysicalNode createServerNode(final String nodename, final NetworkInfrastructure nis) {
    PhysicalNetworkInterface _createPhysicalNetworkInterface = this._masterFactory.createPhysicalNetworkInterface();
    final Procedure1<PhysicalNetworkInterface> _function = new Procedure1<PhysicalNetworkInterface>() {
      @Override
      public void apply(final PhysicalNetworkInterface it) {
        String _genId = NetworkTrafficModelInstanceGenerator.this.genId();
        it.setId(_genId);
        it.setName("eth0");
        NetworkConfiguration _configuration = nis.getConfiguration();
        ProtocolsRepository _protocolsAndStacks = _configuration.getProtocolsAndStacks();
        EList<ProtocolStack> _stacks = _protocolsAndStacks.getStacks();
        final Function1<ProtocolStack, Boolean> _function = new Function1<ProtocolStack, Boolean>() {
          @Override
          public Boolean apply(final ProtocolStack it) {
            String _name = it.getName();
            return Boolean.valueOf(_name.startsWith("TCP"));
          }
        };
        ProtocolStack _findFirst = IterableExtensions.<ProtocolStack>findFirst(_stacks, _function);
        it.setUsedProtocolStack(_findFirst);
        PhysicalNetworkInterfacePerfSpec _createPhysicalNetworkInterfacePerfSpec = NetworkTrafficModelInstanceGenerator.this._masterFactory.createPhysicalNetworkInterfacePerfSpec();
        final Procedure1<PhysicalNetworkInterfacePerfSpec> _function_1 = new Procedure1<PhysicalNetworkInterfacePerfSpec>() {
          @Override
          public void apply(final PhysicalNetworkInterfacePerfSpec it) {
            it.setIsUp(true);
            ConstantDoubleVariable _createConstantDoubleVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
            final Procedure1<ConstantDoubleVariable> _function = new Procedure1<ConstantDoubleVariable>() {
              @Override
              public void apply(final ConstantDoubleVariable it) {
                it.setValue(0.00001);
              }
            };
            ConstantDoubleVariable _doubleArrow = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable, _function);
            it.setPacketProcessingTime(((Dependency) _doubleArrow));
            it.setMTU(1500);
            SpeedUnit _createSpeedUnit = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createSpeedUnit();
            final Procedure1<SpeedUnit> _function_1 = new Procedure1<SpeedUnit>() {
              @Override
              public void apply(final SpeedUnit it) {
                it.setValue(1);
                it.setPrefix(UnitPrefix.G);
                it.setUnit(Speed.BITS_PER_SEC);
              }
            };
            SpeedUnit _doubleArrow_1 = ObjectExtensions.<SpeedUnit>operator_doubleArrow(_createSpeedUnit, _function_1);
            it.setInterfaceSpeed(_doubleArrow_1);
          }
        };
        PhysicalNetworkInterfacePerfSpec _doubleArrow = ObjectExtensions.<PhysicalNetworkInterfacePerfSpec>operator_doubleArrow(_createPhysicalNetworkInterfacePerfSpec, _function_1);
        it.setPerformance(_doubleArrow);
        EList<EntityAddress> _addresses = it.getAddresses();
        EntityAddress _createEntityAddress = NetworkTrafficModelInstanceGenerator.this._masterFactory.createEntityAddress();
        final Procedure1<EntityAddress> _function_2 = new Procedure1<EntityAddress>() {
          @Override
          public void apply(final EntityAddress it) {
            it.setAddress("00:AC:EF:AC:E2:FA:CF");
            NetworkConfiguration _configuration = nis.getConfiguration();
            ProtocolsRepository _protocolsAndStacks = _configuration.getProtocolsAndStacks();
            EList<NetworkProtocol> _protocols = _protocolsAndStacks.getProtocols();
            final Function1<NetworkProtocol, Boolean> _function = new Function1<NetworkProtocol, Boolean>() {
              @Override
              public Boolean apply(final NetworkProtocol it) {
                String _name = it.getName();
                return Boolean.valueOf(_name.startsWith("ETH"));
              }
            };
            NetworkProtocol _findFirst = IterableExtensions.<NetworkProtocol>findFirst(_protocols, _function);
            it.setAddressGivenBy(_findFirst);
          }
        };
        EntityAddress _doubleArrow_1 = ObjectExtensions.<EntityAddress>operator_doubleArrow(_createEntityAddress, _function_2);
        _addresses.add(_doubleArrow_1);
      }
    };
    final PhysicalNetworkInterface netif = ObjectExtensions.<PhysicalNetworkInterface>operator_doubleArrow(_createPhysicalNetworkInterface, _function);
    PhysicalNode _createPhysicalNode = this._masterFactory.createPhysicalNode();
    final Procedure1<PhysicalNode> _function_1 = new Procedure1<PhysicalNode>() {
      @Override
      public void apply(final PhysicalNode it) {
        String _genId = NetworkTrafficModelInstanceGenerator.this.genId();
        it.setId(_genId);
        it.setName(nodename);
        EndNodePerfSpec _createEndNodePerfSpec = NetworkTrafficModelInstanceGenerator.this._masterFactory.createEndNodePerfSpec();
        final Procedure1<EndNodePerfSpec> _function = new Procedure1<EndNodePerfSpec>() {
          @Override
          public void apply(final EndNodePerfSpec it) {
            ConstantDoubleVariable _createConstantDoubleVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
            final Procedure1<ConstantDoubleVariable> _function = new Procedure1<ConstantDoubleVariable>() {
              @Override
              public void apply(final ConstantDoubleVariable it) {
                it.setValue(0.00001);
              }
            };
            ConstantDoubleVariable _doubleArrow = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable, _function);
            it.setSoftwareLayersDelay(_doubleArrow);
            DataPlanePerfSpec _createDataPlanePerfSpec = NetworkTrafficModelInstanceGenerator.this._masterFactory.createDataPlanePerfSpec();
            final Procedure1<DataPlanePerfSpec> _function_1 = new Procedure1<DataPlanePerfSpec>() {
              @Override
              public void apply(final DataPlanePerfSpec it) {
                ConstantDoubleVariable _createConstantDoubleVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                final Procedure1<ConstantDoubleVariable> _function = new Procedure1<ConstantDoubleVariable>() {
                  @Override
                  public void apply(final ConstantDoubleVariable it) {
                    it.setValue(0.0000001);
                  }
                };
                ConstantDoubleVariable _doubleArrow = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable, _function);
                it.setForwardingLatency(_doubleArrow);
                ConstantLongVariable _createConstantLongVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantLongVariable();
                final Procedure1<ConstantLongVariable> _function_1 = new Procedure1<ConstantLongVariable>() {
                  @Override
                  public void apply(final ConstantLongVariable it) {
                    it.setValue(100000000);
                  }
                };
                ConstantLongVariable _doubleArrow_1 = ObjectExtensions.<ConstantLongVariable>operator_doubleArrow(_createConstantLongVariable, _function_1);
                it.setForwardingBandwidthPPS(_doubleArrow_1);
                ConstantLongVariable _createConstantLongVariable_1 = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantLongVariable();
                final Procedure1<ConstantLongVariable> _function_2 = new Procedure1<ConstantLongVariable>() {
                  @Override
                  public void apply(final ConstantLongVariable it) {
                    it.setValue(1000000000);
                  }
                };
                ConstantLongVariable _doubleArrow_2 = ObjectExtensions.<ConstantLongVariable>operator_doubleArrow(_createConstantLongVariable_1, _function_2);
                it.setForwardingBandwidthBPS(_doubleArrow_2);
                ConstantDoubleVariable _createConstantDoubleVariable_1 = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                final Procedure1<ConstantDoubleVariable> _function_3 = new Procedure1<ConstantDoubleVariable>() {
                  @Override
                  public void apply(final ConstantDoubleVariable it) {
                    it.setValue(0.00001);
                  }
                };
                ConstantDoubleVariable _doubleArrow_3 = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable_1, _function_3);
                it.setPacketProcessingTime(_doubleArrow_3);
              }
            };
            DataPlanePerfSpec _doubleArrow_1 = ObjectExtensions.<DataPlanePerfSpec>operator_doubleArrow(_createDataPlanePerfSpec, _function_1);
            it.setDataplane(_doubleArrow_1);
            ControlPlanePerfSpec _createControlPlanePerfSpec = NetworkTrafficModelInstanceGenerator.this._masterFactory.createControlPlanePerfSpec();
            final Procedure1<ControlPlanePerfSpec> _function_2 = new Procedure1<ControlPlanePerfSpec>() {
              @Override
              public void apply(final ControlPlanePerfSpec it) {
                ConstantDoubleVariable _createConstantDoubleVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                final Procedure1<ConstantDoubleVariable> _function = new Procedure1<ConstantDoubleVariable>() {
                  @Override
                  public void apply(final ConstantDoubleVariable it) {
                    it.setValue(0.00001);
                  }
                };
                ConstantDoubleVariable _doubleArrow = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable, _function);
                it.setProcessingLatency(_doubleArrow);
                ConstantLongVariable _createConstantLongVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantLongVariable();
                final Procedure1<ConstantLongVariable> _function_1 = new Procedure1<ConstantLongVariable>() {
                  @Override
                  public void apply(final ConstantLongVariable it) {
                    it.setValue(100000000);
                  }
                };
                ConstantLongVariable _doubleArrow_1 = ObjectExtensions.<ConstantLongVariable>operator_doubleArrow(_createConstantLongVariable, _function_1);
                it.setProcessingBandwidthReqPS(_doubleArrow_1);
                ConstantDoubleVariable _createConstantDoubleVariable_1 = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                final Procedure1<ConstantDoubleVariable> _function_2 = new Procedure1<ConstantDoubleVariable>() {
                  @Override
                  public void apply(final ConstantDoubleVariable it) {
                    it.setValue(0.00001);
                  }
                };
                ConstantDoubleVariable _doubleArrow_2 = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable_1, _function_2);
                it.setPacketProcessingTime(_doubleArrow_2);
              }
            };
            ControlPlanePerfSpec _doubleArrow_2 = ObjectExtensions.<ControlPlanePerfSpec>operator_doubleArrow(_createControlPlanePerfSpec, _function_2);
            it.setControlplane(_doubleArrow_2);
          }
        };
        EndNodePerfSpec _doubleArrow = ObjectExtensions.<EndNodePerfSpec>operator_doubleArrow(_createEndNodePerfSpec, _function);
        it.setPerformance(_doubleArrow);
        EList<PhysicalNetworkInterface> _interfaces = it.getInterfaces();
        _interfaces.add(netif);
      }
    };
    final PhysicalNode node = ObjectExtensions.<PhysicalNode>operator_doubleArrow(_createPhysicalNode, _function_1);
    return node;
  }
  
  public LinkedList<GenericFlow> extraxtFlows(final CaseStudy cs, final IPPair wavelet_flow, final SoftwareComponent destinationSC, final SoftwareComponent sourceSC) {
    LinkedList<GenericFlow> _xblockexpression = null;
    {
      final HashSet<Integer> uniqueDataSizes = new HashSet<Integer>();
      final LinkedList<GenericFlow> flows = new LinkedList<GenericFlow>();
      List<Generator> _generators = wavelet_flow.getGenerators();
      for (final Generator generator : _generators) {
        int _value = generator.getValue();
        uniqueDataSizes.add(Integer.valueOf(_value));
      }
      for (final Integer uds : uniqueDataSizes) {
        {
          GenericFlow _createGenericFlow = this._masterFactory.createGenericFlow();
          final Procedure1<GenericFlow> _function = new Procedure1<GenericFlow>() {
            @Override
            public void apply(final GenericFlow it) {
              String _genId = NetworkTrafficModelInstanceGenerator.this.genId();
              it.setId(_genId);
              it.setDestinationSoftwareComponent(destinationSC);
              it.setSourceSoftwareComponent(sourceSC);
              ConstantLongVariable _createConstantLongVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantLongVariable();
              final Procedure1<ConstantLongVariable> _function = new Procedure1<ConstantLongVariable>() {
                @Override
                public void apply(final ConstantLongVariable it) {
                  it.setValue((uds).intValue());
                }
              };
              ConstantLongVariable _doubleArrow = ObjectExtensions.<ConstantLongVariable>operator_doubleArrow(_createConstantLongVariable, _function);
              it.setDataSize(((Dependency) _doubleArrow));
              String _name = wavelet_flow.getName();
              String _plus = (_name + "_size_");
              String _plus_1 = (_plus + uds);
              it.setName(_plus_1);
              EList<String> _alternativeNames = it.getAlternativeNames();
              String _name_1 = wavelet_flow.getName();
              String _plus_2 = ("wavelet_gen_id_" + _name_1);
              _alternativeNames.add(_plus_2);
            }
          };
          final GenericFlow dniflow = ObjectExtensions.<GenericFlow>operator_doubleArrow(_createGenericFlow, _function);
          flows.add(dniflow);
        }
      }
      _xblockexpression = flows;
    }
    return _xblockexpression;
  }
  
  public NetworkTraffic extraxtSoftwareComponents(final CaseStudy cs, final NetworkInfrastructure nis) {
    InputOutput.<String>println("==== Creating Software components");
    List<IPPair> _ippairs = cs.getIppairs();
    for (final IPPair ippair : _ippairs) {
      {
        String _name = ippair.getName();
        String _plus = ("== Processing ippair: " + _name);
        String _plus_1 = (_plus + ". it has ");
        List<Generator> _generators = ippair.getGenerators();
        int _size = _generators.size();
        String _plus_2 = (_plus_1 + Integer.valueOf(_size));
        String _plus_3 = (_plus_2 + " generators");
        InputOutput.<String>println(_plus_3);
        SoftwareComponent _createSoftwareComponent = this._masterFactory.createSoftwareComponent();
        final Procedure1<SoftwareComponent> _function = new Procedure1<SoftwareComponent>() {
          @Override
          public void apply(final SoftwareComponent s) {
            String _genId = NetworkTrafficModelInstanceGenerator.this.genId();
            s.setId(_genId);
            NetworkStructure _structure = nis.getStructure();
            EList<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _nodes = _structure.getNodes();
            final Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean> _function = new Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean>() {
              @Override
              public Boolean apply(final Node<NetworkInterface<NetworkInterfacePerfSpec>> it) {
                String _name = it.getName();
                String _rightIP = NetworkTrafficModelInstanceGenerator.this.getRightIP(ippair);
                return Boolean.valueOf(_name.endsWith(_rightIP));
              }
            };
            final Iterable<Node<NetworkInterface<NetworkInterfacePerfSpec>>> candidate_nodes = IterableExtensions.<Node<NetworkInterface<NetworkInterfacePerfSpec>>>filter(_nodes, _function);
            String _rightIP = NetworkTrafficModelInstanceGenerator.this.getRightIP(ippair);
            String _plus = ("destination_w_ip_" + _rightIP);
            s.setName(_plus);
            int _size = IterableExtensions.size(candidate_nodes);
            boolean _greaterThan = (_size > 0);
            if (_greaterThan) {
              Node<NetworkInterface<NetworkInterfacePerfSpec>> _head = IterableExtensions.<Node<NetworkInterface<NetworkInterfacePerfSpec>>>head(candidate_nodes);
              s.setDeployedOn(((Node) _head));
              String _name = s.getName();
              String _plus_1 = ("#### SoftComponent " + _name);
              String _plus_2 = (_plus_1 + " ID ");
              String _plus_3 = (_plus_2 + Integer.valueOf(NetworkTrafficModelInstanceGenerator.this.id));
              String _plus_4 = (_plus_3 + " will be deployed on node ");
              Node<NetworkInterface<NetworkInterfacePerfSpec>> _deployedOn = s.getDeployedOn();
              String _plus_5 = (_plus_4 + _deployedOn);
              InputOutput.<String>println(_plus_5);
            } else {
              String _name_1 = s.getName();
              String _plus_6 = ("#### SoftComponent " + _name_1);
              String _plus_7 = (_plus_6 + " ID ");
              String _plus_8 = (_plus_7 + Integer.valueOf(NetworkTrafficModelInstanceGenerator.this.id));
              String _plus_9 = (_plus_8 + " NO CANDIDATE NODE TO DEPLOY ON");
              InputOutput.<String>println(_plus_9);
            }
          }
        };
        final SoftwareComponent destinationSoftwareComponent = ObjectExtensions.<SoftwareComponent>operator_doubleArrow(_createSoftwareComponent, _function);
        String _name_1 = destinationSoftwareComponent.getName();
        String _plus_4 = ("Creating Software component " + _name_1);
        InputOutput.<String>println(_plus_4);
        SoftwareComponent _createSoftwareComponent_1 = this._masterFactory.createSoftwareComponent();
        final Procedure1<SoftwareComponent> _function_1 = new Procedure1<SoftwareComponent>() {
          @Override
          public void apply(final SoftwareComponent it) {
            String _genId = NetworkTrafficModelInstanceGenerator.this.genId();
            it.setId(_genId);
            NetworkStructure _structure = nis.getStructure();
            EList<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _nodes = _structure.getNodes();
            final Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean> _function = new Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean>() {
              @Override
              public Boolean apply(final Node<NetworkInterface<NetworkInterfacePerfSpec>> it) {
                String _name = it.getName();
                String _leftIP = NetworkTrafficModelInstanceGenerator.this.getLeftIP(ippair);
                return Boolean.valueOf(_name.endsWith(_leftIP));
              }
            };
            final Iterable<Node<NetworkInterface<NetworkInterfacePerfSpec>>> candidate_nodes = IterableExtensions.<Node<NetworkInterface<NetworkInterfacePerfSpec>>>filter(_nodes, _function);
            int _size = IterableExtensions.size(candidate_nodes);
            boolean _greaterThan = (_size > 0);
            if (_greaterThan) {
              Node<NetworkInterface<NetworkInterfacePerfSpec>> _head = IterableExtensions.<Node<NetworkInterface<NetworkInterfacePerfSpec>>>head(candidate_nodes);
              it.setDeployedOn(((Node) _head));
              String _name = it.getName();
              String _plus = ("#### SoftComponent " + _name);
              String _plus_1 = (_plus + " ID ");
              String _plus_2 = (_plus_1 + Integer.valueOf(NetworkTrafficModelInstanceGenerator.this.id));
              String _plus_3 = (_plus_2 + " will be deployed on node ");
              Node<NetworkInterface<NetworkInterfacePerfSpec>> _deployedOn = it.getDeployedOn();
              String _plus_4 = (_plus_3 + _deployedOn);
              InputOutput.<String>println(_plus_4);
            }
            String _leftIP = NetworkTrafficModelInstanceGenerator.this.getLeftIP(ippair);
            String _plus_5 = ("source_w_ip_" + _leftIP);
            it.setName(_plus_5);
          }
        };
        final SoftwareComponent sourceSoftwareComponent = ObjectExtensions.<SoftwareComponent>operator_doubleArrow(_createSoftwareComponent_1, _function_1);
        String _name_2 = sourceSoftwareComponent.getName();
        String _plus_5 = ("Creating Software component " + _name_2);
        InputOutput.<String>println(_plus_5);
        final LinkedList<GenericFlow> dni_flows = this.extraxtFlows(cs, ippair, destinationSoftwareComponent, sourceSoftwareComponent);
        for (final GenericFlow dniflow : dni_flows) {
          {
            String _name_3 = dniflow.getName();
            String _plus_6 = ("\t Creating Flow " + _name_3);
            InputOutput.<String>println(_plus_6);
            NetworkTraffic _traffic = nis.getTraffic();
            EList<Flow> _flows = _traffic.getFlows();
            _flows.add(dniflow);
          }
        }
        TrafficSource _createTrafficSource = this._masterFactory.createTrafficSource();
        final Procedure1<TrafficSource> _function_2 = new Procedure1<TrafficSource>() {
          @Override
          public void apply(final TrafficSource it) {
            String _genId = NetworkTrafficModelInstanceGenerator.this.genId();
            it.setId(_genId);
            it.setName("empty");
            it.setSoftwareComponent(sourceSoftwareComponent);
            EList<Workload> _workload = it.getWorkload();
            GenericWorkload _createGenericWorkload = NetworkTrafficModelInstanceGenerator.this._masterFactory.createGenericWorkload();
            _workload.add(_createGenericWorkload);
          }
        };
        final TrafficSource ts = ObjectExtensions.<TrafficSource>operator_doubleArrow(_createTrafficSource, _function_2);
        EList<TrafficSource> _trafficSources = sourceSoftwareComponent.getTrafficSources();
        _trafficSources.add(ts);
        NetworkTraffic _traffic = nis.getTraffic();
        EList<SoftwareComponent> _software = _traffic.getSoftware();
        _software.add(destinationSoftwareComponent);
        NetworkTraffic _traffic_1 = nis.getTraffic();
        EList<SoftwareComponent> _software_1 = _traffic_1.getSoftware();
        _software_1.add(sourceSoftwareComponent);
        InputOutput.<String>println("== Software component added");
      }
    }
    InputOutput.<String>println("==== Creating Software components done");
    return nis.getTraffic();
  }
  
  public String extraxtWorkloads(final CaseStudy cs, final NetworkInfrastructure nis) {
    String _xblockexpression = null;
    {
      InputOutput.<String>println("==== Creating Workloads");
      List<IPPair> _ippairs = cs.getIppairs();
      final Function1<IPPair, Boolean> _function = new Function1<IPPair, Boolean>() {
        @Override
        public Boolean apply(final IPPair it) {
          List<Generator> _generators = it.getGenerators();
          int _size = _generators.size();
          return Boolean.valueOf((_size > 0));
        }
      };
      Iterable<IPPair> _filter = IterableExtensions.<IPPair>filter(_ippairs, _function);
      for (final IPPair ippair : _filter) {
        {
          final String sender_cs_ip = this.getLeftIP(ippair);
          final String receiver_cs_ip = this.getRightIP(ippair);
          NetworkTraffic _traffic = nis.getTraffic();
          EList<Flow> _flows = _traffic.getFlows();
          final Function1<Flow, Boolean> _function_1 = new Function1<Flow, Boolean>() {
            @Override
            public Boolean apply(final Flow it) {
              boolean _and = false;
              String _name = it.getName();
              String _iPOutOfFlowName = NetworkTrafficModelInstanceGenerator.this.getIPOutOfFlowName(0, _name);
              boolean _equals = Objects.equal(_iPOutOfFlowName, sender_cs_ip);
              if (!_equals) {
                _and = false;
              } else {
                String _name_1 = it.getName();
                String _iPOutOfFlowName_1 = NetworkTrafficModelInstanceGenerator.this.getIPOutOfFlowName(1, _name_1);
                boolean _equals_1 = Objects.equal(_iPOutOfFlowName_1, receiver_cs_ip);
                _and = _equals_1;
              }
              return Boolean.valueOf(_and);
            }
          };
          final Iterable<Flow> example_flows = IterableExtensions.<Flow>filter(_flows, _function_1);
          final Flow example_flow = ((Flow[])Conversions.unwrapArray(example_flows, Flow.class))[0];
          final SoftwareComponent sender_softwareComponent = example_flow.getSourceSoftwareComponent();
          final SoftwareComponent receiver_softwareComponent = example_flow.getDestinationSoftwareComponent();
          InputOutput.<String>println("\t ==== Checking software components");
          boolean _or = false;
          EList<TrafficSource> _trafficSources = sender_softwareComponent.getTrafficSources();
          int _size = _trafficSources.size();
          boolean _greaterThan = (_size > 1);
          if (_greaterThan) {
            _or = true;
          } else {
            EList<TrafficSource> _trafficSources_1 = sender_softwareComponent.getTrafficSources();
            boolean _equals = Objects.equal(_trafficSources_1, Integer.valueOf(0));
            _or = _equals;
          }
          if (_or) {
            InputOutput.<String>println("!!!ALARM: TODO! soft component has no tses or too many tses");
          }
          EList<TrafficSource> _trafficSources_2 = sender_softwareComponent.getTrafficSources();
          final TrafficSource ts = _trafficSources_2.get(0);
          EList<Workload> _workload = ts.getWorkload();
          int _size_1 = _workload.size();
          boolean _notEquals = (_size_1 != 1);
          if (_notEquals) {
            InputOutput.<String>println("!!!ALARM: TODO! TS has more than one workload");
          }
          EList<Workload> _workload_1 = ts.getWorkload();
          Workload _get = _workload_1.get(0);
          final GenericWorkload wrk = ((GenericWorkload) _get);
          EList<AbstractAction> _actions = wrk.getActions();
          int _size_2 = _actions.size();
          boolean _notEquals_1 = (_size_2 != 0);
          if (_notEquals_1) {
            InputOutput.<String>println("!!!ALARM: TODO! TS has workload with existing actions!");
          }
          InputOutput.<String>println("\t ==== Checking software components done");
          InputOutput.<String>println("\t ==== Extracting workload");
          final BranchAction branch = this._masterFactory.createBranchAction();
          EList<AbstractAction> _actions_1 = wrk.getActions();
          SequenceAction _createSequenceAction = this._masterFactory.createSequenceAction();
          final Procedure1<SequenceAction> _function_2 = new Procedure1<SequenceAction>() {
            @Override
            public void apply(final SequenceAction it) {
              EList<AbstractAction> _intern = it.getIntern();
              StartAction _createStartAction = NetworkTrafficModelInstanceGenerator.this._masterFactory.createStartAction();
              _intern.add(_createStartAction);
              EList<AbstractAction> _intern_1 = it.getIntern();
              _intern_1.add(branch);
              EList<AbstractAction> _intern_2 = it.getIntern();
              StopAction _createStopAction = NetworkTrafficModelInstanceGenerator.this._masterFactory.createStopAction();
              _intern_2.add(_createStopAction);
            }
          };
          SequenceAction _doubleArrow = ObjectExtensions.<SequenceAction>operator_doubleArrow(_createSequenceAction, _function_2);
          _actions_1.add(_doubleArrow);
          List<Generator> _generators = ippair.getGenerators();
          for (final Generator gen : _generators) {
            {
              InputOutput.<String>println("\t --------------------");
              InputOutput.<String>println(("\t Processing gen: " + gen));
              final Function1<Flow, Boolean> _function_3 = new Function1<Flow, Boolean>() {
                @Override
                public Boolean apply(final Flow it) {
                  String _name = it.getName();
                  String _name_1 = ippair.getName();
                  String _plus = (_name_1 + "_size_");
                  int _value = gen.getValue();
                  String _plus_1 = (_plus + Integer.valueOf(_value));
                  return Boolean.valueOf(Objects.equal(_name, _plus_1));
                }
              };
              Iterable<Flow> _filter_1 = IterableExtensions.<Flow>filter(example_flows, _function_3);
              final Flow the_flow = ((Flow[])Conversions.unwrapArray(_filter_1, Flow.class))[0];
              String _name = the_flow.getName();
              String _plus = ("\t Found flow: " + _name);
              InputOutput.<String>println(_plus);
              WaitAction _createWaitAction = this._masterFactory.createWaitAction();
              final Procedure1<WaitAction> _function_4 = new Procedure1<WaitAction>() {
                @Override
                public void apply(final WaitAction it) {
                  ConstantDoubleVariable _createConstantDoubleVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                  final Procedure1<ConstantDoubleVariable> _function = new Procedure1<ConstantDoubleVariable>() {
                    @Override
                    public void apply(final ConstantDoubleVariable it) {
                      double _begin = gen.getBegin();
                      it.setValue(_begin);
                    }
                  };
                  ConstantDoubleVariable _doubleArrow = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable, _function);
                  it.setWaitTime(((Dependency) _doubleArrow));
                }
              };
              final WaitAction initial_wait = ObjectExtensions.<WaitAction>operator_doubleArrow(_createWaitAction, _function_4);
              LoopAction _createLoopAction = this._masterFactory.createLoopAction();
              final Procedure1<LoopAction> _function_5 = new Procedure1<LoopAction>() {
                @Override
                public void apply(final LoopAction it) {
                  SequenceAction _createSequenceAction = NetworkTrafficModelInstanceGenerator.this._masterFactory.createSequenceAction();
                  final Procedure1<SequenceAction> _function = new Procedure1<SequenceAction>() {
                    @Override
                    public void apply(final SequenceAction it) {
                      EList<AbstractAction> _intern = it.getIntern();
                      TransmitAction _createTransmitAction = NetworkTrafficModelInstanceGenerator.this._masterFactory.createTransmitAction();
                      final Procedure1<TransmitAction> _function = new Procedure1<TransmitAction>() {
                        @Override
                        public void apply(final TransmitAction it) {
                          it.setFlow(the_flow);
                        }
                      };
                      TransmitAction _doubleArrow = ObjectExtensions.<TransmitAction>operator_doubleArrow(_createTransmitAction, _function);
                      _intern.add(_doubleArrow);
                      EList<AbstractAction> _intern_1 = it.getIntern();
                      WaitAction _createWaitAction = NetworkTrafficModelInstanceGenerator.this._masterFactory.createWaitAction();
                      final Procedure1<WaitAction> _function_1 = new Procedure1<WaitAction>() {
                        @Override
                        public void apply(final WaitAction it) {
                          ConstantDoubleVariable _createConstantDoubleVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                          final Procedure1<ConstantDoubleVariable> _function = new Procedure1<ConstantDoubleVariable>() {
                            @Override
                            public void apply(final ConstantDoubleVariable it) {
                              int _scale = gen.getScale();
                              double _pow = Math.pow(2, _scale);
                              it.setValue(_pow);
                            }
                          };
                          ConstantDoubleVariable _doubleArrow = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable, _function);
                          it.setWaitTime(((Dependency) _doubleArrow));
                        }
                      };
                      WaitAction _doubleArrow_1 = ObjectExtensions.<WaitAction>operator_doubleArrow(_createWaitAction, _function_1);
                      _intern_1.add(_doubleArrow_1);
                    }
                  };
                  SequenceAction _doubleArrow = ObjectExtensions.<SequenceAction>operator_doubleArrow(_createSequenceAction, _function);
                  it.setIntern(_doubleArrow);
                  final long calculated_num_iter = NetworkTrafficModelInstanceGenerator.this.calculateNumIterations(gen);
                  ConstantLongVariable _createConstantLongVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantLongVariable();
                  final Procedure1<ConstantLongVariable> _function_1 = new Procedure1<ConstantLongVariable>() {
                    @Override
                    public void apply(final ConstantLongVariable it) {
                      it.setValue(calculated_num_iter);
                    }
                  };
                  ConstantLongVariable _doubleArrow_1 = ObjectExtensions.<ConstantLongVariable>operator_doubleArrow(_createConstantLongVariable, _function_1);
                  it.setNumIterations(((Dependency) _doubleArrow_1));
                }
              };
              final LoopAction loop = ObjectExtensions.<LoopAction>operator_doubleArrow(_createLoopAction, _function_5);
              SequenceAction _createSequenceAction_1 = this._masterFactory.createSequenceAction();
              final Procedure1<SequenceAction> _function_6 = new Procedure1<SequenceAction>() {
                @Override
                public void apply(final SequenceAction it) {
                  EList<AbstractAction> _intern = it.getIntern();
                  _intern.add(initial_wait);
                  EList<AbstractAction> _intern_1 = it.getIntern();
                  _intern_1.add(loop);
                }
              };
              final SequenceAction seq = ObjectExtensions.<SequenceAction>operator_doubleArrow(_createSequenceAction_1, _function_6);
              EList<AbstractAction> _intern = branch.getIntern();
              _intern.add(seq);
            }
          }
          InputOutput.<String>println("\t ==== Extracting workload done");
        }
      }
      _xblockexpression = InputOutput.<String>println("==== Creating Workloads done");
    }
    return _xblockexpression;
  }
  
  public long calculateNumIterations(final Generator gen) {
    double _end = gen.getEnd();
    double _begin = gen.getBegin();
    final double duration = (_end - _begin);
    int _scale = gen.getScale();
    final double interval = Math.pow(2, _scale);
    if ((interval > duration)) {
      return ((long) 1);
    }
    if ((interval == duration)) {
      return ((long) 2);
    }
    final double numIter = (1 + (((double) duration) / ((double) interval)));
    return ((long) numIter);
  }
  
  public String buildTheRest(final CaseStudy study, final NetworkInfrastructure infrastructure) {
    String _xblockexpression = null;
    {
      NetworkConfiguration _configuration = this.ns.getConfiguration();
      RoutesRepository _createRoutesRepository = this._masterFactory.createRoutesRepository();
      _configuration.setRoutes(_createRoutesRepository);
      this.buildTopology(infrastructure);
      _xblockexpression = this.buildRouting(infrastructure);
    }
    return _xblockexpression;
  }
  
  public String buildRouting(final NetworkInfrastructure inf) {
    String _xblockexpression = null;
    {
      InputOutput.<String>println("==== Building Routes");
      NetworkTraffic _traffic = inf.getTraffic();
      final EList<Flow> flows = _traffic.getFlows();
      for (final Flow f : flows) {
        {
          final SoftwareComponent sourceSoftware = f.getSourceSoftwareComponent();
          final EList<TrafficSource> trafficSources = sourceSoftware.getTrafficSources();
          boolean _and = false;
          int _size = trafficSources.size();
          boolean _greaterThan = (_size > 0);
          if (!_greaterThan) {
            _and = false;
          } else {
            TrafficSource _head = IterableExtensions.<TrafficSource>head(trafficSources);
            EList<Workload> _workload = _head.getWorkload();
            int _size_1 = _workload.size();
            boolean _greaterThan_1 = (_size_1 > 0);
            _and = _greaterThan_1;
          }
          if (_and) {
            final Node<NetworkInterface<NetworkInterfacePerfSpec>> sourceNode = sourceSoftware.getDeployedOn();
            SoftwareComponent _destinationSoftwareComponent = f.getDestinationSoftwareComponent();
            final Node<NetworkInterface<NetworkInterfacePerfSpec>> destinationNode = _destinationSoftwareComponent.getDeployedOn();
            final Route route = this.createRouteForNodes(inf, sourceNode, destinationNode);
            NetworkConfiguration _configuration = inf.getConfiguration();
            RoutesRepository _routes = _configuration.getRoutes();
            EList<Route> _routes_1 = _routes.getRoutes();
            _routes_1.add(route);
          }
        }
      }
      NetworkConfiguration _configuration = inf.getConfiguration();
      RoutesRepository _routes = _configuration.getRoutes();
      EList<Route> _routes_1 = _routes.getRoutes();
      Set<Route> r1s = IterableExtensions.<Route>toSet(_routes_1);
      Set<Route> toRemove = new HashSet<Route>();
      for (final Route r1 : r1s) {
        for (final Route r2 : r1s) {
          boolean _and = false;
          boolean _and_1 = false;
          boolean _and_2 = false;
          String _id = r1.getId();
          String _id_1 = r2.getId();
          boolean _notEquals = (!Objects.equal(_id, _id_1));
          if (!_notEquals) {
            _and_2 = false;
          } else {
            boolean _routesIdentic = this.routesIdentic(r1, r2);
            _and_2 = _routesIdentic;
          }
          if (!_and_2) {
            _and_1 = false;
          } else {
            boolean _contains = toRemove.contains(r1);
            boolean _not = (!_contains);
            _and_1 = _not;
          }
          if (!_and_1) {
            _and = false;
          } else {
            boolean _contains_1 = toRemove.contains(r2);
            boolean _not_1 = (!_contains_1);
            _and = _not_1;
          }
          if (_and) {
            String _id_2 = r1.getId();
            String _plus = ("\t Removing duplicate route " + _id_2);
            String _plus_1 = (_plus + " - is identic with ");
            String _id_3 = r2.getId();
            String _plus_2 = (_plus_1 + _id_3);
            InputOutput.<String>println(_plus_2);
            toRemove.add(r1);
          }
        }
      }
      for (final Route r : toRemove) {
        NetworkConfiguration _configuration_1 = inf.getConfiguration();
        RoutesRepository _routes_2 = _configuration_1.getRoutes();
        EList<Route> _routes_3 = _routes_2.getRoutes();
        _routes_3.remove(r);
      }
      _xblockexpression = InputOutput.<String>println("==== Building Routes done");
    }
    return _xblockexpression;
  }
  
  public boolean routesIdentic(final Route r1, final Route r2) {
    NetworkInterface<?> _start = r1.getStart();
    String _id = _start.getId();
    NetworkInterface<?> _start_1 = r2.getStart();
    String _id_1 = _start_1.getId();
    boolean _notEquals = (!Objects.equal(_id, _id_1));
    if (_notEquals) {
      return false;
    }
    NetworkInterface<?> _end = r1.getEnd();
    String _id_2 = _end.getId();
    NetworkInterface<?> _end_1 = r2.getEnd();
    String _id_3 = _end_1.getId();
    boolean _notEquals_1 = (!Objects.equal(_id_2, _id_3));
    if (_notEquals_1) {
      return false;
    }
    boolean _and = false;
    Hop _hops = r1.getHops();
    boolean _notEquals_2 = (!Objects.equal(_hops, null));
    if (!_notEquals_2) {
      _and = false;
    } else {
      Hop _hops_1 = r2.getHops();
      boolean _notEquals_3 = (!Objects.equal(_hops_1, null));
      _and = _notEquals_3;
    }
    if (_and) {
      Hop hop1 = r1.getHops();
      Hop hop2 = r2.getHops();
      NetworkInterface<NetworkInterfacePerfSpec> _interfaceref = hop1.getInterfaceref();
      String _id_4 = _interfaceref.getId();
      NetworkInterface<NetworkInterfacePerfSpec> _interfaceref_1 = hop2.getInterfaceref();
      String _id_5 = _interfaceref_1.getId();
      boolean _notEquals_4 = (!Objects.equal(_id_4, _id_5));
      if (_notEquals_4) {
        return false;
      }
      while (((!Objects.equal(hop1.getNextHop(), null)) && (!Objects.equal(hop2.getNextHop(), null)))) {
        {
          Hop _nextHop = hop1.getNextHop();
          hop1 = _nextHop;
          Hop _nextHop_1 = hop2.getNextHop();
          hop2 = _nextHop_1;
          NetworkInterface<NetworkInterfacePerfSpec> _interfaceref_2 = hop1.getInterfaceref();
          String _id_6 = _interfaceref_2.getId();
          NetworkInterface<NetworkInterfacePerfSpec> _interfaceref_3 = hop2.getInterfaceref();
          String _id_7 = _interfaceref_3.getId();
          boolean _notEquals_5 = (!Objects.equal(_id_6, _id_7));
          if (_notEquals_5) {
            return false;
          }
        }
      }
      boolean _or = false;
      boolean _and_1 = false;
      Hop _nextHop = hop1.getNextHop();
      boolean _equals = Objects.equal(_nextHop, null);
      if (!_equals) {
        _and_1 = false;
      } else {
        Hop _nextHop_1 = hop2.getNextHop();
        boolean _notEquals_5 = (!Objects.equal(_nextHop_1, null));
        _and_1 = _notEquals_5;
      }
      if (_and_1) {
        _or = true;
      } else {
        boolean _and_2 = false;
        Hop _nextHop_2 = hop1.getNextHop();
        boolean _notEquals_6 = (!Objects.equal(_nextHop_2, null));
        if (!_notEquals_6) {
          _and_2 = false;
        } else {
          Hop _nextHop_3 = hop2.getNextHop();
          boolean _equals_1 = Objects.equal(_nextHop_3, null);
          _and_2 = _equals_1;
        }
        _or = _and_2;
      }
      if (_or) {
        return false;
      }
      boolean _and_3 = false;
      boolean _and_4 = false;
      boolean _and_5 = false;
      boolean _and_6 = false;
      Hop _nextHop_4 = hop1.getNextHop();
      boolean _equals_2 = Objects.equal(_nextHop_4, null);
      if (!_equals_2) {
        _and_6 = false;
      } else {
        Hop _nextHop_5 = hop2.getNextHop();
        boolean _equals_3 = Objects.equal(_nextHop_5, null);
        _and_6 = _equals_3;
      }
      if (!_and_6) {
        _and_5 = false;
      } else {
        NetworkInterface<NetworkInterfacePerfSpec> _interfaceref_2 = hop1.getInterfaceref();
        String _id_6 = _interfaceref_2.getId();
        NetworkInterface<?> _end_2 = r1.getEnd();
        String _id_7 = _end_2.getId();
        boolean _equals_4 = Objects.equal(_id_6, _id_7);
        _and_5 = _equals_4;
      }
      if (!_and_5) {
        _and_4 = false;
      } else {
        NetworkInterface<NetworkInterfacePerfSpec> _interfaceref_3 = hop2.getInterfaceref();
        String _id_8 = _interfaceref_3.getId();
        NetworkInterface<?> _end_3 = r2.getEnd();
        String _id_9 = _end_3.getId();
        boolean _equals_5 = Objects.equal(_id_8, _id_9);
        _and_4 = _equals_5;
      }
      if (!_and_4) {
        _and_3 = false;
      } else {
        NetworkInterface<?> _end_4 = r1.getEnd();
        String _id_10 = _end_4.getId();
        NetworkInterface<?> _end_5 = r2.getEnd();
        String _id_11 = _end_5.getId();
        boolean _equals_6 = Objects.equal(_id_10, _id_11);
        _and_3 = _equals_6;
      }
      if (_and_3) {
        return true;
      }
    }
    InputOutput.<String>println("\t shall never get here");
    return false;
  }
  
  public Route createRouteForNodes(final NetworkInfrastructure inf, final Node startNode, final Node endNode) {
    NetworkInterface if1 = null;
    NetworkInterface if2 = null;
    EList _interfaces = startNode.getInterfaces();
    final Function1<NetworkInterface, Boolean> _function = new Function1<NetworkInterface, Boolean>() {
      @Override
      public Boolean apply(final NetworkInterface i) {
        Object _performance = i.getPerformance();
        boolean _isIsUp = ((NetworkInterfacePerfSpec) _performance).isIsUp();
        return Boolean.valueOf((_isIsUp == true));
      }
    };
    final Iterable<NetworkInterface> startNodeIfaces = IterableExtensions.<NetworkInterface>filter(_interfaces, _function);
    EList _interfaces_1 = endNode.getInterfaces();
    final Function1<NetworkInterface, Boolean> _function_1 = new Function1<NetworkInterface, Boolean>() {
      @Override
      public Boolean apply(final NetworkInterface i) {
        Object _performance = i.getPerformance();
        boolean _isIsUp = ((NetworkInterfacePerfSpec) _performance).isIsUp();
        return Boolean.valueOf((_isIsUp == true));
      }
    };
    final Iterable<NetworkInterface> endNodeIfaces = IterableExtensions.<NetworkInterface>filter(_interfaces_1, _function_1);
    boolean _and = false;
    int _size = IterableExtensions.size(startNodeIfaces);
    boolean _equals = (_size == 1);
    if (!_equals) {
      _and = false;
    } else {
      int _size_1 = IterableExtensions.size(endNodeIfaces);
      boolean _equals_1 = (_size_1 == 1);
      _and = _equals_1;
    }
    if (_and) {
      Route _createRoute = this._masterFactory.createRoute();
      final Procedure1<Route> _function_2 = new Procedure1<Route>() {
        @Override
        public void apply(final Route it) {
          NetworkInterface _head = IterableExtensions.<NetworkInterface>head(startNodeIfaces);
          it.setStart(_head);
          NetworkInterface _head_1 = IterableExtensions.<NetworkInterface>head(endNodeIfaces);
          it.setEnd(_head_1);
          it.setName("Route");
          String _genId = NetworkTrafficModelInstanceGenerator.this.genId();
          it.setId(_genId);
          EList<String> _alternativeNames = it.getAlternativeNames();
          String _name = startNode.getName();
          String _plus = ("Route_from_node_" + _name);
          String _plus_1 = (_plus + "_if_");
          NetworkInterface _head_2 = IterableExtensions.<NetworkInterface>head(startNodeIfaces);
          String _name_1 = _head_2.getName();
          String _plus_2 = (_plus_1 + _name_1);
          String _plus_3 = (_plus_2 + "_to_");
          String _name_2 = endNode.getName();
          String _plus_4 = (_plus_3 + _name_2);
          String _plus_5 = (_plus_4 + "_if_");
          NetworkInterface _head_3 = IterableExtensions.<NetworkInterface>head(endNodeIfaces);
          String _name_3 = _head_3.getName();
          String _plus_6 = (_plus_5 + _name_3);
          _alternativeNames.add(_plus_6);
          EList<String> _alternativeNames_1 = it.getAlternativeNames();
          String _head_4 = IterableExtensions.<String>head(_alternativeNames_1);
          it.setName(_head_4);
        }
      };
      final Route route = ObjectExtensions.<Route>operator_doubleArrow(_createRoute, _function_2);
      NetworkStructure _structure = inf.getStructure();
      EList<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>> _links = _structure.getLinks();
      final Function1<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>, Boolean> _function_3 = new Function1<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>, Boolean>() {
        @Override
        public Boolean apply(final Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>> l) {
          EList<NetworkInterface<NetworkInterfacePerfSpec>> _connects = l.getConnects();
          NetworkInterface _head = IterableExtensions.<NetworkInterface>head(startNodeIfaces);
          return Boolean.valueOf(_connects.contains(_head));
        }
      };
      Iterable<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>> _filter = IterableExtensions.<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>>filter(_links, _function_3);
      final Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>> kabel = IterableExtensions.<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>>head(_filter);
      final Hop hop = this._masterFactory.createHop();
      boolean _notEquals = (!Objects.equal(kabel, null));
      if (_notEquals) {
        EList<NetworkInterface<NetworkInterfacePerfSpec>> _connects = kabel.getConnects();
        final NetworkInterface<NetworkInterfacePerfSpec> end1 = _connects.get(0);
        EList<NetworkInterface<NetworkInterfacePerfSpec>> _connects_1 = kabel.getConnects();
        final NetworkInterface<NetworkInterfacePerfSpec> end2 = _connects_1.get(1);
        NetworkInterface nextIface = end1;
        NetworkInterface _head = IterableExtensions.<NetworkInterface>head(startNodeIfaces);
        boolean _equals_2 = Objects.equal(end1, _head);
        if (_equals_2) {
          nextIface = end2;
        }
        boolean _and_1 = false;
        NetworkInterface _head_1 = IterableExtensions.<NetworkInterface>head(startNodeIfaces);
        boolean _equals_3 = Objects.equal(end1, _head_1);
        if (!_equals_3) {
          _and_1 = false;
        } else {
          NetworkInterface _head_2 = IterableExtensions.<NetworkInterface>head(endNodeIfaces);
          boolean _equals_4 = Objects.equal(end2, _head_2);
          _and_1 = _equals_4;
        }
        if (_and_1) {
          Hop _createHop = this._masterFactory.createHop();
          final Procedure1<Hop> _function_4 = new Procedure1<Hop>() {
            @Override
            public void apply(final Hop it) {
              it.setInterfaceref(end2);
            }
          };
          Hop _doubleArrow = ObjectExtensions.<Hop>operator_doubleArrow(_createHop, _function_4);
          route.setHops(_doubleArrow);
          return route;
        }
        boolean _and_2 = false;
        NetworkInterface _head_3 = IterableExtensions.<NetworkInterface>head(startNodeIfaces);
        boolean _equals_5 = Objects.equal(end2, _head_3);
        if (!_equals_5) {
          _and_2 = false;
        } else {
          NetworkInterface _head_4 = IterableExtensions.<NetworkInterface>head(endNodeIfaces);
          boolean _equals_6 = Objects.equal(end1, _head_4);
          _and_2 = _equals_6;
        }
        if (_and_2) {
          Hop _createHop_1 = this._masterFactory.createHop();
          final Procedure1<Hop> _function_5 = new Procedure1<Hop>() {
            @Override
            public void apply(final Hop it) {
              it.setInterfaceref(end1);
            }
          };
          Hop _doubleArrow_1 = ObjectExtensions.<Hop>operator_doubleArrow(_createHop_1, _function_5);
          route.setHops(_doubleArrow_1);
          return route;
        }
        hop.setInterfaceref(nextIface);
        route.setHops(hop);
      }
      NetworkStructure _structure_1 = inf.getStructure();
      EList<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>> _links_1 = _structure_1.getLinks();
      final Function1<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>, Boolean> _function_6 = new Function1<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>, Boolean>() {
        @Override
        public Boolean apply(final Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>> l) {
          EList<NetworkInterface<NetworkInterfacePerfSpec>> _connects = l.getConnects();
          NetworkInterface _head = IterableExtensions.<NetworkInterface>head(endNodeIfaces);
          return Boolean.valueOf(_connects.contains(_head));
        }
      };
      Iterable<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>> _filter_1 = IterableExtensions.<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>>filter(_links_1, _function_6);
      final Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>> endKabel = IterableExtensions.<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>>head(_filter_1);
      boolean _notEquals_1 = (!Objects.equal(endKabel, null));
      if (_notEquals_1) {
        EList<NetworkInterface<NetworkInterfacePerfSpec>> _connects_2 = endKabel.getConnects();
        final NetworkInterface<NetworkInterfacePerfSpec> end1_1 = _connects_2.get(0);
        EList<NetworkInterface<NetworkInterfacePerfSpec>> _connects_3 = endKabel.getConnects();
        final NetworkInterface<NetworkInterfacePerfSpec> end2_1 = _connects_3.get(1);
        NetworkInterface outgoingSwitchInterface = end1_1;
        NetworkInterface _head_5 = IterableExtensions.<NetworkInterface>head(endNodeIfaces);
        boolean _equals_7 = Objects.equal(end1_1, _head_5);
        if (_equals_7) {
          outgoingSwitchInterface = end2_1;
        }
        boolean _equals_8 = Objects.equal(outgoingSwitchInterface, end1_1);
        if (_equals_8) {
          Hop _hops = route.getHops();
          Hop _createHop_2 = this._masterFactory.createHop();
          final Procedure1<Hop> _function_7 = new Procedure1<Hop>() {
            @Override
            public void apply(final Hop it) {
              it.setInterfaceref(end1_1);
            }
          };
          Hop _doubleArrow_2 = ObjectExtensions.<Hop>operator_doubleArrow(_createHop_2, _function_7);
          _hops.setNextHop(_doubleArrow_2);
          Hop _hops_1 = route.getHops();
          Hop _nextHop = _hops_1.getNextHop();
          Hop _createHop_3 = this._masterFactory.createHop();
          final Procedure1<Hop> _function_8 = new Procedure1<Hop>() {
            @Override
            public void apply(final Hop it) {
              it.setInterfaceref(end2_1);
            }
          };
          Hop _doubleArrow_3 = ObjectExtensions.<Hop>operator_doubleArrow(_createHop_3, _function_8);
          _nextHop.setNextHop(_doubleArrow_3);
        } else {
          Hop _hops_2 = route.getHops();
          Hop _createHop_4 = this._masterFactory.createHop();
          final Procedure1<Hop> _function_9 = new Procedure1<Hop>() {
            @Override
            public void apply(final Hop it) {
              it.setInterfaceref(end2_1);
            }
          };
          Hop _doubleArrow_4 = ObjectExtensions.<Hop>operator_doubleArrow(_createHop_4, _function_9);
          _hops_2.setNextHop(_doubleArrow_4);
          Hop _hops_3 = route.getHops();
          Hop _nextHop_1 = _hops_3.getNextHop();
          Hop _createHop_5 = this._masterFactory.createHop();
          final Procedure1<Hop> _function_10 = new Procedure1<Hop>() {
            @Override
            public void apply(final Hop it) {
              it.setInterfaceref(end1_1);
            }
          };
          Hop _doubleArrow_5 = ObjectExtensions.<Hop>operator_doubleArrow(_createHop_5, _function_10);
          _nextHop_1.setNextHop(_doubleArrow_5);
        }
        return route;
      }
    }
    InputOutput.<String>println("returning empty route");
    return this._masterFactory.createRoute();
  }
  
  public String buildTopology(final NetworkInfrastructure inf) {
    String _xblockexpression = null;
    {
      InputOutput.<String>println("==== Building Topology");
      NetworkStructure _structure = inf.getStructure();
      EList<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _nodes = _structure.getNodes();
      final Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean> _function = new Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean>() {
        @Override
        public Boolean apply(final Node<NetworkInterface<NetworkInterfacePerfSpec>> node) {
          PerformanceSpecification _performance = node.getPerformance();
          Class<? extends PerformanceSpecification> _class = _performance.getClass();
          return Boolean.valueOf(Objects.equal(_class, EndNodePerfSpecImpl.class));
        }
      };
      final Iterable<Node<NetworkInterface<NetworkInterfacePerfSpec>>> servers = IterableExtensions.<Node<NetworkInterface<NetworkInterfacePerfSpec>>>filter(_nodes, _function);
      NetworkStructure _structure_1 = inf.getStructure();
      EList<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _nodes_1 = _structure_1.getNodes();
      final Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean> _function_1 = new Function1<Node<NetworkInterface<NetworkInterfacePerfSpec>>, Boolean>() {
        @Override
        public Boolean apply(final Node<NetworkInterface<NetworkInterfacePerfSpec>> node) {
          PerformanceSpecification _performance = node.getPerformance();
          Class<? extends PerformanceSpecification> _class = _performance.getClass();
          return Boolean.valueOf(Objects.equal(_class, IntermediateNodePerfSpecImpl.class));
        }
      };
      final Iterable<Node<NetworkInterface<NetworkInterfacePerfSpec>>> switches = IterableExtensions.<Node<NetworkInterface<NetworkInterfacePerfSpec>>>filter(_nodes_1, _function_1);
      PhysicalNode _createPhysicalNode = this._masterFactory.createPhysicalNode();
      final Procedure1<PhysicalNode> _function_2 = new Procedure1<PhysicalNode>() {
        @Override
        public void apply(final PhysicalNode it) {
          String _genId = NetworkTrafficModelInstanceGenerator.this.genId();
          it.setId(_genId);
          it.setName("theSwitch");
          IntermediateNodePerfSpec _createIntermediateNodePerfSpec = NetworkTrafficModelInstanceGenerator.this._masterFactory.createIntermediateNodePerfSpec();
          final Procedure1<IntermediateNodePerfSpec> _function = new Procedure1<IntermediateNodePerfSpec>() {
            @Override
            public void apply(final IntermediateNodePerfSpec it) {
              DataPlanePerfSpec _createDataPlanePerfSpec = NetworkTrafficModelInstanceGenerator.this._masterFactory.createDataPlanePerfSpec();
              final Procedure1<DataPlanePerfSpec> _function = new Procedure1<DataPlanePerfSpec>() {
                @Override
                public void apply(final DataPlanePerfSpec it) {
                  ConstantDoubleVariable _createConstantDoubleVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                  final Procedure1<ConstantDoubleVariable> _function = new Procedure1<ConstantDoubleVariable>() {
                    @Override
                    public void apply(final ConstantDoubleVariable it) {
                      it.setValue(0.0000001);
                    }
                  };
                  ConstantDoubleVariable _doubleArrow = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable, _function);
                  it.setForwardingLatency(_doubleArrow);
                  ConstantLongVariable _createConstantLongVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantLongVariable();
                  final Procedure1<ConstantLongVariable> _function_1 = new Procedure1<ConstantLongVariable>() {
                    @Override
                    public void apply(final ConstantLongVariable it) {
                      it.setValue(100000000);
                    }
                  };
                  ConstantLongVariable _doubleArrow_1 = ObjectExtensions.<ConstantLongVariable>operator_doubleArrow(_createConstantLongVariable, _function_1);
                  it.setForwardingBandwidthPPS(_doubleArrow_1);
                  ConstantLongVariable _createConstantLongVariable_1 = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantLongVariable();
                  final Procedure1<ConstantLongVariable> _function_2 = new Procedure1<ConstantLongVariable>() {
                    @Override
                    public void apply(final ConstantLongVariable it) {
                      it.setValue(1000000000);
                    }
                  };
                  ConstantLongVariable _doubleArrow_2 = ObjectExtensions.<ConstantLongVariable>operator_doubleArrow(_createConstantLongVariable_1, _function_2);
                  it.setForwardingBandwidthBPS(_doubleArrow_2);
                  ConstantDoubleVariable _createConstantDoubleVariable_1 = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                  final Procedure1<ConstantDoubleVariable> _function_3 = new Procedure1<ConstantDoubleVariable>() {
                    @Override
                    public void apply(final ConstantDoubleVariable it) {
                      it.setValue(0.00001);
                    }
                  };
                  ConstantDoubleVariable _doubleArrow_3 = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable_1, _function_3);
                  it.setPacketProcessingTime(_doubleArrow_3);
                }
              };
              DataPlanePerfSpec _doubleArrow = ObjectExtensions.<DataPlanePerfSpec>operator_doubleArrow(_createDataPlanePerfSpec, _function);
              it.setDataplane(_doubleArrow);
              ControlPlanePerfSpec _createControlPlanePerfSpec = NetworkTrafficModelInstanceGenerator.this._masterFactory.createControlPlanePerfSpec();
              final Procedure1<ControlPlanePerfSpec> _function_1 = new Procedure1<ControlPlanePerfSpec>() {
                @Override
                public void apply(final ControlPlanePerfSpec it) {
                  ConstantDoubleVariable _createConstantDoubleVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                  final Procedure1<ConstantDoubleVariable> _function = new Procedure1<ConstantDoubleVariable>() {
                    @Override
                    public void apply(final ConstantDoubleVariable it) {
                      it.setValue(0.00001);
                    }
                  };
                  ConstantDoubleVariable _doubleArrow = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable, _function);
                  it.setProcessingLatency(_doubleArrow);
                  ConstantLongVariable _createConstantLongVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantLongVariable();
                  final Procedure1<ConstantLongVariable> _function_1 = new Procedure1<ConstantLongVariable>() {
                    @Override
                    public void apply(final ConstantLongVariable it) {
                      it.setValue(100000000);
                    }
                  };
                  ConstantLongVariable _doubleArrow_1 = ObjectExtensions.<ConstantLongVariable>operator_doubleArrow(_createConstantLongVariable, _function_1);
                  it.setProcessingBandwidthReqPS(_doubleArrow_1);
                  ConstantDoubleVariable _createConstantDoubleVariable_1 = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                  final Procedure1<ConstantDoubleVariable> _function_2 = new Procedure1<ConstantDoubleVariable>() {
                    @Override
                    public void apply(final ConstantDoubleVariable it) {
                      it.setValue(0.00001);
                    }
                  };
                  ConstantDoubleVariable _doubleArrow_2 = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable_1, _function_2);
                  it.setPacketProcessingTime(_doubleArrow_2);
                }
              };
              ControlPlanePerfSpec _doubleArrow_1 = ObjectExtensions.<ControlPlanePerfSpec>operator_doubleArrow(_createControlPlanePerfSpec, _function_1);
              it.setControlplane(_doubleArrow_1);
            }
          };
          IntermediateNodePerfSpec _doubleArrow = ObjectExtensions.<IntermediateNodePerfSpec>operator_doubleArrow(_createIntermediateNodePerfSpec, _function);
          it.setPerformance(_doubleArrow);
        }
      };
      final PhysicalNode starSw = ObjectExtensions.<PhysicalNode>operator_doubleArrow(_createPhysicalNode, _function_2);
      NetworkStructure _structure_2 = inf.getStructure();
      EList<Node<NetworkInterface<NetworkInterfacePerfSpec>>> _nodes_2 = _structure_2.getNodes();
      _nodes_2.add(((Node) starSw));
      int i = 0;
      for (final Node<NetworkInterface<NetworkInterfacePerfSpec>> s : servers) {
        {
          final PhysicalNetworkInterface interface_ = this.createPreconfiguredPhysicalNetworkInterface(inf, i);
          EList<NetworkInterface<NetworkInterfacePerfSpec>> _interfaces = s.getInterfaces();
          NetworkInterface<NetworkInterfacePerfSpec> _head = IterableExtensions.<NetworkInterface<NetworkInterfacePerfSpec>>head(_interfaces);
          final NetworkInterface serverInterface = ((NetworkInterface) _head);
          EList<PhysicalNetworkInterface> _interfaces_1 = starSw.getInterfaces();
          _interfaces_1.add(interface_);
          PhysicalLink _createPhysicalLink = this._masterFactory.createPhysicalLink();
          final Procedure1<PhysicalLink> _function_3 = new Procedure1<PhysicalLink>() {
            @Override
            public void apply(final PhysicalLink it) {
              String _genId = NetworkTrafficModelInstanceGenerator.this.genId();
              it.setId(_genId);
              String _id = starSw.getId();
              String _plus = ("kabel_switch_" + _id);
              String _plus_1 = (_plus + "_node_");
              String _id_1 = s.getId();
              String _plus_2 = (_plus_1 + _id_1);
              it.setName(_plus_2);
              EList<PhysicalNetworkInterface> _connects = it.getConnects();
              _connects.add(((PhysicalNetworkInterface) serverInterface));
              EList<PhysicalNetworkInterface> _connects_1 = it.getConnects();
              _connects_1.add(interface_);
              PhysicalLinkPerfSpec _createPhysicalLinkPerfSpec = NetworkTrafficModelInstanceGenerator.this._masterFactory.createPhysicalLinkPerfSpec();
              final Procedure1<PhysicalLinkPerfSpec> _function = new Procedure1<PhysicalLinkPerfSpec>() {
                @Override
                public void apply(final PhysicalLinkPerfSpec it) {
                  ConstantDoubleVariable _createConstantDoubleVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
                  final Procedure1<ConstantDoubleVariable> _function = new Procedure1<ConstantDoubleVariable>() {
                    @Override
                    public void apply(final ConstantDoubleVariable it) {
                      it.setValue(5.7e-9);
                    }
                  };
                  ConstantDoubleVariable _doubleArrow = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable, _function);
                  it.setPropagationDelay(_doubleArrow);
                }
              };
              PhysicalLinkPerfSpec _doubleArrow = ObjectExtensions.<PhysicalLinkPerfSpec>operator_doubleArrow(_createPhysicalLinkPerfSpec, _function);
              it.setPerformance(_doubleArrow);
            }
          };
          PhysicalLink link = ObjectExtensions.<PhysicalLink>operator_doubleArrow(_createPhysicalLink, _function_3);
          NetworkStructure _structure_3 = inf.getStructure();
          EList<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>> _links = _structure_3.getLinks();
          _links.add(((Link) link));
          i = (i + 1);
        }
      }
      InputOutput.<String>println(("\tServers: " + servers));
      InputOutput.<String>println(("\tSwitches: " + switches));
      NetworkStructure _structure_3 = inf.getStructure();
      EList<Link<LinkPerfSpec, NetworkInterface<NetworkInterfacePerfSpec>>> _links = _structure_3.getLinks();
      String _plus = ("\tLinks: " + _links);
      InputOutput.<String>println(_plus);
      _xblockexpression = InputOutput.<String>println("==== Building Topology done");
    }
    return _xblockexpression;
  }
  
  public PhysicalNetworkInterface createPreconfiguredPhysicalNetworkInterface(final NetworkInfrastructure inf, final int index) {
    PhysicalNetworkInterface _createPhysicalNetworkInterface = this._masterFactory.createPhysicalNetworkInterface();
    final Procedure1<PhysicalNetworkInterface> _function = new Procedure1<PhysicalNetworkInterface>() {
      @Override
      public void apply(final PhysicalNetworkInterface it) {
        String _genId = NetworkTrafficModelInstanceGenerator.this.genId();
        it.setId(_genId);
        it.setName(("eth" + Integer.valueOf(index)));
        NetworkConfiguration _configuration = inf.getConfiguration();
        ProtocolsRepository _protocolsAndStacks = _configuration.getProtocolsAndStacks();
        EList<ProtocolStack> _stacks = _protocolsAndStacks.getStacks();
        final Function1<ProtocolStack, Boolean> _function = new Function1<ProtocolStack, Boolean>() {
          @Override
          public Boolean apply(final ProtocolStack it) {
            String _name = it.getName();
            return Boolean.valueOf(_name.startsWith("TCP"));
          }
        };
        ProtocolStack _findFirst = IterableExtensions.<ProtocolStack>findFirst(_stacks, _function);
        it.setUsedProtocolStack(_findFirst);
        PhysicalNetworkInterfacePerfSpec _createPhysicalNetworkInterfacePerfSpec = NetworkTrafficModelInstanceGenerator.this._masterFactory.createPhysicalNetworkInterfacePerfSpec();
        final Procedure1<PhysicalNetworkInterfacePerfSpec> _function_1 = new Procedure1<PhysicalNetworkInterfacePerfSpec>() {
          @Override
          public void apply(final PhysicalNetworkInterfacePerfSpec it) {
            it.setIsUp(true);
            ConstantDoubleVariable _createConstantDoubleVariable = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createConstantDoubleVariable();
            final Procedure1<ConstantDoubleVariable> _function = new Procedure1<ConstantDoubleVariable>() {
              @Override
              public void apply(final ConstantDoubleVariable it) {
                it.setValue(0.00001);
              }
            };
            ConstantDoubleVariable _doubleArrow = ObjectExtensions.<ConstantDoubleVariable>operator_doubleArrow(_createConstantDoubleVariable, _function);
            it.setPacketProcessingTime(_doubleArrow);
            it.setMTU(1500);
            SpeedUnit _createSpeedUnit = NetworkTrafficModelInstanceGenerator.this._unitsFactory.createSpeedUnit();
            final Procedure1<SpeedUnit> _function_1 = new Procedure1<SpeedUnit>() {
              @Override
              public void apply(final SpeedUnit it) {
                it.setValue(1);
                it.setPrefix(UnitPrefix.G);
                it.setUnit(Speed.BITS_PER_SEC);
              }
            };
            SpeedUnit _doubleArrow_1 = ObjectExtensions.<SpeedUnit>operator_doubleArrow(_createSpeedUnit, _function_1);
            it.setInterfaceSpeed(_doubleArrow_1);
          }
        };
        PhysicalNetworkInterfacePerfSpec _doubleArrow = ObjectExtensions.<PhysicalNetworkInterfacePerfSpec>operator_doubleArrow(_createPhysicalNetworkInterfacePerfSpec, _function_1);
        it.setPerformance(_doubleArrow);
        EList<EntityAddress> _addresses = it.getAddresses();
        EntityAddress _createEntityAddress = NetworkTrafficModelInstanceGenerator.this._masterFactory.createEntityAddress();
        final Procedure1<EntityAddress> _function_2 = new Procedure1<EntityAddress>() {
          @Override
          public void apply(final EntityAddress it) {
            it.setAddress("00:AC:EF:AC:E2:FA:CF");
            NetworkConfiguration _configuration = inf.getConfiguration();
            ProtocolsRepository _protocolsAndStacks = _configuration.getProtocolsAndStacks();
            EList<NetworkProtocol> _protocols = _protocolsAndStacks.getProtocols();
            final Function1<NetworkProtocol, Boolean> _function = new Function1<NetworkProtocol, Boolean>() {
              @Override
              public Boolean apply(final NetworkProtocol it) {
                String _name = it.getName();
                return Boolean.valueOf(_name.startsWith("ETH"));
              }
            };
            NetworkProtocol _findFirst = IterableExtensions.<NetworkProtocol>findFirst(_protocols, _function);
            it.setAddressGivenBy(_findFirst);
          }
        };
        EntityAddress _doubleArrow_1 = ObjectExtensions.<EntityAddress>operator_doubleArrow(_createEntityAddress, _function_2);
        _addresses.add(_doubleArrow_1);
      }
    };
    return ObjectExtensions.<PhysicalNetworkInterface>operator_doubleArrow(_createPhysicalNetworkInterface, _function);
  }
  
  public void addProtocols(final NetworkConfiguration conf) {
    ProtocolsRepository _createProtocolsRepository = this._masterFactory.createProtocolsRepository();
    final Procedure1<ProtocolsRepository> _function = new Procedure1<ProtocolsRepository>() {
      @Override
      public void apply(final ProtocolsRepository it) {
        ProtocolLayer _createProtocolLayer = NetworkTrafficModelInstanceGenerator.this._masterFactory.createProtocolLayer();
        final Procedure1<ProtocolLayer> _function = new Procedure1<ProtocolLayer>() {
          @Override
          public void apply(final ProtocolLayer it) {
            NetworkProtocol _createNetworkProtocol = NetworkTrafficModelInstanceGenerator.this._masterFactory.createNetworkProtocol();
            final Procedure1<NetworkProtocol> _function = new Procedure1<NetworkProtocol>() {
              @Override
              public void apply(final NetworkProtocol it) {
                it.setName("ETH");
                it.setDataUnitLength(1500);
                it.setHeadersLength(32);
                it.setConnectionless(true);
                it.setDeliveryGuaranteed(false);
              }
            };
            NetworkProtocol _doubleArrow = ObjectExtensions.<NetworkProtocol>operator_doubleArrow(_createNetworkProtocol, _function);
            it.setProtocol(_doubleArrow);
            NetworkProtocol _protocol = it.getProtocol();
            String _name = _protocol.getName();
            it.setName(_name);
          }
        };
        final ProtocolLayer layerETH = ObjectExtensions.<ProtocolLayer>operator_doubleArrow(_createProtocolLayer, _function);
        ProtocolLayer _createProtocolLayer_1 = NetworkTrafficModelInstanceGenerator.this._masterFactory.createProtocolLayer();
        final Procedure1<ProtocolLayer> _function_1 = new Procedure1<ProtocolLayer>() {
          @Override
          public void apply(final ProtocolLayer it) {
            NetworkProtocol _createNetworkProtocol = NetworkTrafficModelInstanceGenerator.this._masterFactory.createNetworkProtocol();
            final Procedure1<NetworkProtocol> _function = new Procedure1<NetworkProtocol>() {
              @Override
              public void apply(final NetworkProtocol it) {
                it.setName("IP");
                it.setDataUnitLength(65000);
                it.setHeadersLength(48);
                it.setConnectionless(true);
                it.setDeliveryGuaranteed(false);
              }
            };
            NetworkProtocol _doubleArrow = ObjectExtensions.<NetworkProtocol>operator_doubleArrow(_createNetworkProtocol, _function);
            it.setProtocol(_doubleArrow);
            NetworkProtocol _protocol = it.getProtocol();
            String _name = _protocol.getName();
            it.setName(_name);
            it.setIsCarriedBy(layerETH);
          }
        };
        final ProtocolLayer layerIP = ObjectExtensions.<ProtocolLayer>operator_doubleArrow(_createProtocolLayer_1, _function_1);
        ProtocolLayer _createProtocolLayer_2 = NetworkTrafficModelInstanceGenerator.this._masterFactory.createProtocolLayer();
        final Procedure1<ProtocolLayer> _function_2 = new Procedure1<ProtocolLayer>() {
          @Override
          public void apply(final ProtocolLayer it) {
            NetworkProtocol _createNetworkProtocol = NetworkTrafficModelInstanceGenerator.this._masterFactory.createNetworkProtocol();
            final Procedure1<NetworkProtocol> _function = new Procedure1<NetworkProtocol>() {
              @Override
              public void apply(final NetworkProtocol it) {
                it.setName("TCP");
                it.setDataUnitLength(65000);
                it.setHeadersLength(48);
                it.setConnectionless(false);
                it.setDeliveryGuaranteed(true);
              }
            };
            NetworkProtocol _doubleArrow = ObjectExtensions.<NetworkProtocol>operator_doubleArrow(_createNetworkProtocol, _function);
            it.setProtocol(_doubleArrow);
            NetworkProtocol _protocol = it.getProtocol();
            String _name = _protocol.getName();
            it.setName(_name);
            it.setIsCarriedBy(layerIP);
          }
        };
        final ProtocolLayer layerTCP = ObjectExtensions.<ProtocolLayer>operator_doubleArrow(_createProtocolLayer_2, _function_2);
        EList<NetworkProtocol> _protocols = it.getProtocols();
        NetworkProtocol _protocol = layerETH.getProtocol();
        _protocols.add(_protocol);
        EList<NetworkProtocol> _protocols_1 = it.getProtocols();
        NetworkProtocol _protocol_1 = layerIP.getProtocol();
        _protocols_1.add(_protocol_1);
        EList<NetworkProtocol> _protocols_2 = it.getProtocols();
        NetworkProtocol _protocol_2 = layerTCP.getProtocol();
        _protocols_2.add(_protocol_2);
        EList<ProtocolStack> _stacks = it.getStacks();
        ProtocolStack _createProtocolStack = NetworkTrafficModelInstanceGenerator.this._masterFactory.createProtocolStack();
        final Procedure1<ProtocolStack> _function_3 = new Procedure1<ProtocolStack>() {
          @Override
          public void apply(final ProtocolStack it) {
            it.setName("TCP/IP/ETH");
            EList<ProtocolLayer> _layers = it.getLayers();
            _layers.add(layerTCP);
            EList<ProtocolLayer> _layers_1 = it.getLayers();
            _layers_1.add(layerIP);
            EList<ProtocolLayer> _layers_2 = it.getLayers();
            _layers_2.add(layerETH);
          }
        };
        ProtocolStack _doubleArrow = ObjectExtensions.<ProtocolStack>operator_doubleArrow(_createProtocolStack, _function_3);
        _stacks.add(_doubleArrow);
        EList<ProtocolStack> _stacks_1 = it.getStacks();
        ProtocolStack _createProtocolStack_1 = NetworkTrafficModelInstanceGenerator.this._masterFactory.createProtocolStack();
        final Procedure1<ProtocolStack> _function_4 = new Procedure1<ProtocolStack>() {
          @Override
          public void apply(final ProtocolStack it) {
            it.setName("IP/ETH");
            EList<ProtocolLayer> _layers = it.getLayers();
            _layers.add(layerIP);
            EList<ProtocolLayer> _layers_1 = it.getLayers();
            _layers_1.add(layerETH);
          }
        };
        ProtocolStack _doubleArrow_1 = ObjectExtensions.<ProtocolStack>operator_doubleArrow(_createProtocolStack_1, _function_4);
        _stacks_1.add(_doubleArrow_1);
        EList<ProtocolStack> _stacks_2 = it.getStacks();
        ProtocolStack _createProtocolStack_2 = NetworkTrafficModelInstanceGenerator.this._masterFactory.createProtocolStack();
        final Procedure1<ProtocolStack> _function_5 = new Procedure1<ProtocolStack>() {
          @Override
          public void apply(final ProtocolStack it) {
            it.setName("ETH");
            EList<ProtocolLayer> _layers = it.getLayers();
            _layers.add(layerETH);
          }
        };
        ProtocolStack _doubleArrow_2 = ObjectExtensions.<ProtocolStack>operator_doubleArrow(_createProtocolStack_2, _function_5);
        _stacks_2.add(_doubleArrow_2);
      }
    };
    ProtocolsRepository _doubleArrow = ObjectExtensions.<ProtocolsRepository>operator_doubleArrow(_createProtocolsRepository, _function);
    conf.setProtocolsAndStacks(_doubleArrow);
  }
}
